"""This module is for the translation mapping data of the
following document:

Document file: index.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# apysc documentation": "# apysc ドキュメント",
    ##################################################
    "Welcome to apysc documentation! apysc is a Python front-end library (currently developing and only works partially).": "apyscのドキュメントページへようこそ。apyscはPythonのフロントエンド用のライブラリです（現在開発中であり部分的にのみ動作します）。",  # noqa
    ##################################################
    "## Project links": "## プロジェクトの関連リンク",
    ##################################################
    "- [GitHub](https://github.com/simon-ritchie/apysc)": "- [GitHub](https://github.com/simon-ritchie/apysc)",  # noqa
    ##################################################
    "  - Please leave a ⭐️star⭐️ if you favor the apysc library or have high hopes for the library's future!": "  - もしもapyscライブラリを気にいったり、もしくはライブラリの将来に期待が出来そうと感じられたらリポジトリにスターを付けていただけますと幸いです。",  # noqa
    ##################################################
    "- [PyPI](https://pypi.org/project/apysc/)": "- [PyPI](https://pypi.org/project/apysc/)",  # noqa
    ##################################################
    "Other languages documentations: | English | [Japanese (日本語)](https://simon-ritchie.github.io/apysc/jp/jp_index.html) |": "他の言語のドキュメント: | [英語 (English)](https://simon-ritchie.github.io/apysc/en/index.html) | 日本語 |",  # noqa
    ##################################################
    "## Quick start guide": "## クイックスタートガイド",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nwhat_apysc_can_do\nquick_start\nimport_conventions\nrecommended_type_checker_settings\n```": "```{toctree}\n:maxdepth: 1\njp_what_apysc_can_do\njp_quick_start\njp_import_conventions\njp_recommended_type_checker_settings\n```",  # noqa
    ##################################################
    "## Container classes": "## コンテナーの各クラス",
    ##################################################
    "The `Stage` is the apysc overall drawing area container, and the `Sprite` is the container class.": "`Stage`はapyscの描画領域全体のコンテナーとなり、`Sprite`は通常の各インスタンスのコンテナーのクラスとなります。",  # noqa
    ##################################################
    "```{toctree}\n:maxdepth: 1\nstage\nsprite\n```": "```{toctree}\n:maxdepth: 1\njp_stage\njp_sprite\n```",  # noqa
    ##################################################
    "## Exporting": "## 出力処理",
    ##################################################
    "The HTML and JavaScript exporting interfaces.": "以下はHTMLとJavaScriptの出力処理関係の各インターフェイスです。",  # noqa
    ##################################################
    "```{toctree}\n:maxdepth: 1\nsave_overall_html\ndisplay_on_jupyter\ndisplay_on_colaboratory\nappend_js_expression\n```": "```{toctree}\n:maxdepth: 1\njp_save_overall_html\njp_display_on_jupyter\njp_display_on_colaboratory\njp_append_js_expression\n```",  # noqa
    ##################################################
    "## Child-related interfaces": "## 子要素関係の各インターフェイス",
    ##################################################
    "The parent class, such as the `Sprite` or `Stage`, has the following interfaces:": "`Sprite`や`Stage`などの親となれる各クラスは以下のインターフェイスを持っています:",  # noqa
    ##################################################
    "```{toctree}\n:maxdepth: 1\nadd_child_and_remove_child\nremove_children\ncontains\nnum_children\nget_child_at\n```": "```{toctree}\n:maxdepth: 1\njp_add_child_and_remove_child\njp_remove_children\njp_contains\njp_num_children\njp_get_child_at\n```",  # noqa
    ##################################################
    "## apysc basic data classes": "## apyscの基本的な各データクラス",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nwhy_apysc_doesnt_use_python_builtin_data_type\nfundamental_data_classes_value_interface\nto_string\n```": "```{toctree}\n:maxdepth: 1\njp_why_apysc_doesnt_use_python_builtin_data_type\njp_fundamental_data_classes_value_interface\njp_to_string\n```",  # noqa
    ##################################################
    "### Int and Number classes": "### Int と Number クラス",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nint_and_number\nint_and_number_arithmetic_operations\nint_and_number_comparison_operations\nint_and_number_to_fixed\nint_and_number_to_hex\n```": "```{toctree}\n:maxdepth: 1\njp_int_and_number\njp_int_and_number_arithmetic_operations\njp_int_and_number_comparison_operations\njp_int_and_number_to_fixed\njp_int_and_number_to_hex\n```",  # noqa
    ##################################################
    "### String class": "### String クラス",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nstring\nstring_comparison_operations\nstring_addition_and_multiplication\nstring_split\nstring_lstrip\nstring_strip\nstring_rstrip\nstring_length\nstring_apply_max_num_of_decimal_places\nstring_zfill\nstring_lower\nstring_upper\nstring_slice\n```": "```{toctree}\n:maxdepth: 1\njp_string\njp_string_comparison_operations\njp_string_addition_and_multiplication\njp_string_split\njp_string_lstrip\njp_string_strip\njp_string_rstrip\njp_string_length\njp_string_apply_max_num_of_decimal_places\njp_string_zfill\njp_string_lower\njp_string_upper\njp_string_slice\n```",  # noqa
    ##################################################
    "### Boolean class and constants": "### Boolean クラスと各定数値",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nboolean\ntrue_and_false\n```": "```{toctree}\n:maxdepth: 1\njp_boolean\njp_true_and_false\n```",  # noqa
    ##################################################
    "### Array class": "### Array クラス",
    ##################################################
    "```{toctree}\n:maxdepth: 1\narray\narray_append_and_push\narray_extend_and_concat\narray_insert_and_insert_at\narray_pop\narray_remove_and_remove_at\narray_sort\narray_reverse\narray_slice\narray_length\narray_join\narray_index_of\narray_clear\narray_comparison\narray_last_value\n```": "```{toctree}\n:maxdepth: 1\njp_array\njp_array_append_and_push\njp_array_extend_and_concat\njp_array_insert_and_insert_at\njp_array_pop\njp_array_remove_and_remove_at\njp_array_sort\njp_array_reverse\njp_array_slice\njp_array_length\njp_array_join\njp_array_index_of\njp_array_clear\njp_array_comparison\njp_array_last_value\n```",  # noqa
    ##################################################
    "### Dictionary class": "### Dictionary クラス",
    ##################################################
    "```{toctree}\n:maxdepth: 1\ndictionary\ndictionary_generic\ndictionary_get\ndictionary_length\n```": "```{toctree}\n:maxdepth: 1\njp_dictionary\njp_dictionary_generic\njp_dictionary_get\njp_dictionary_length\n```",  # noqa
    ##################################################
    "### Color class and constants": "### Color クラスと各定数",
    ##################################################
    "```{toctree}\n:maxdepth: 1\ncolor\ncolors\nmaterial_design_colors\ncolorless\ncolor_from_rgb\nred_color\ngreen_color\nblue_color\n```": "```{toctree}\n:maxdepth: 1\njp_color\njp_colors\njp_material_design_colors\njp_colorless\njp_color_from_rgb\njp_red_color\njp_green_color\njp_blue_color\n```",  # noqa
    ##################################################
    "## DisplayObject and GraphicsBase classes": "## DisplayObject と GraphicsBase の各クラス",  # noqa
    ##################################################
    "The `DisplayObject` class is the base class for each display object. The `GraphicsBase` class is the `DisplayObject` subclass and the base class of each graphic class, such as the `Rectangle` class.": "`DisplayObject`クラスは各表示用のオブジェクトの基底クラスです。`GraphicsBase`クラスは`DisplayObject`のサブクラスであり、且つ`Rectangle`などの各グラフィックスの基底クラスとなります。",  # noqa
    ##################################################
    "```{toctree}\n:maxdepth: 1\ndisplay_object\ndisplay_object_and_graphics_base_prop_abstract\ndisplay_object_x_and_y\ndisplay_object_parent\ndisplay_object_visible\ndisplay_object_get_and_set_css\ndisplay_object_mouse_event\ngraphics_base_fill_color\ngraphics_base_fill_alpha\ngraphics_base_line_color\ngraphics_base_line_alpha\ngraphics_base_line_thickness\ngraphics_base_line_dot_setting\ngraphics_base_line_dash_setting\ngraphics_base_line_round_dot_setting\ngraphics_base_line_dash_dot_setting\ngraphics_base_rotation_around_center\ngraphics_base_rotation_around_point\ngraphics_base_scale_from_center\ngraphics_base_scale_from_point\ngraphics_base_flip_interfaces\ngraphics_base_skew\n```": "```{toctree}\n:maxdepth: 1\njp_display_object\njp_display_object_and_graphics_base_prop_abstract\njp_display_object_x_and_y\njp_display_object_parent\njp_display_object_visible\njp_display_object_get_and_set_css\njp_display_object_mouse_event\njp_graphics_base_fill_color\njp_graphics_base_fill_alpha\njp_graphics_base_line_color\njp_graphics_base_line_alpha\njp_graphics_base_line_thickness\njp_graphics_base_line_dot_setting\njp_graphics_base_line_dash_setting\njp_graphics_base_line_round_dot_setting\njp_graphics_base_line_dash_dot_setting\njp_graphics_base_rotation_around_center\njp_graphics_base_rotation_around_point\njp_graphics_base_scale_from_center\njp_graphics_base_scale_from_point\njp_graphics_base_flip_interfaces\njp_graphics_base_skew\n```",  # noqa
    ##################################################
    "## Each graphic class": "## 各グラフィックのクラス",
    ##################################################
    "```{toctree}\n:maxdepth: 1\ntriangle\nrectangle\ncircle\nellipse\nline\npolyline\npolygon\npath\n```": "```{toctree}\n:maxdepth: 1\njp_triangle\njp_rectangle\njp_circle\njp_ellipse\njp_line\njp_polyline\njp_polygon\njp_path\n```",  # noqa
    ##################################################
    "## Graphics class": "## Graphics クラス",
    ##################################################
    "The `Graphics` class handles each vector graphics drawing.": "`Graphics`クラスは各ベクターグラフィックスの描画処理を扱います。",  # noqa
    ##################################################
    "```{toctree}\n:maxdepth: 1\ndraw_interfaces_abstract\ngraphics\ngraphics_begin_fill\ngraphics_line_style\ngraphics_draw_triangle\ngraphics_draw_rect\ngraphics_draw_round_rect\ngraphics_draw_circle\ngraphics_draw_ellipse\ngraphics_move_to_and_line_to\ngraphics_draw_line\ngraphics_draw_dotted_line\ngraphics_draw_dashed_line\ngraphics_draw_round_dotted_line\ngraphics_draw_dash_dotted_line\ngraphics_draw_polygon\ngraphics_draw_path\ngraphics_clear\n```": "```{toctree}\n:maxdepth: 1\njp_draw_interfaces_abstract\njp_graphics\njp_graphics_begin_fill\njp_graphics_line_style\njp_graphics_draw_triangle\njp_graphics_draw_rect\njp_graphics_draw_round_rect\njp_graphics_draw_circle\njp_graphics_draw_ellipse\njp_graphics_move_to_and_line_to\njp_graphics_draw_line\njp_graphics_draw_dotted_line\njp_graphics_draw_dashed_line\njp_graphics_draw_round_dotted_line\njp_graphics_draw_dash_dotted_line\njp_graphics_draw_polygon\njp_graphics_draw_path\njp_graphics_clear\n```",  # noqa
    ##################################################
    "## Text": "## テキスト",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nmulti_line_text\ntext_font_size\ntext_fill_color\ntext_fill_alpha\ntext_bold\ntext_italic\ntext_align\ntext_align_last\ntext_underline\ntext_line_height\n```": "```{toctree}\n:maxdepth: 1\njp_multi_line_text\njp_text_font_size\njp_text_fill_color\njp_text_fill_alpha\njp_text_bold\njp_text_italic\njp_text_align\njp_text_align_last\njp_text_underline\njp_text_line_height\n```",  # noqa
    ##################################################
    "## SVG text": "## SVG テキスト",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nsvg_text\nsvg_text_span\n```": "```{toctree}\n:maxdepth: 1\njp_svg_text\njp_svg_text_span\n```",  # noqa
    ##################################################
    "## Geometry-related classes": "## 座標・サイズ等のデータの各クラス",
    ##################################################
    "```{toctree}\n:maxdepth: 1\npoint2d\npath_move_to\npath_line_to\npath_horizontal\npath_vertical\npath_close\npath_bezier_2d\npath_bezier_2d_continual\npath_bezier_3d\npath_bezier_3d_continual\nrectangle_geom\nget_bounds\n```": "```{toctree}\n:maxdepth: 1\njp_point2d\njp_path_move_to\njp_path_line_to\njp_path_horizontal\njp_path_vertical\njp_path_close\njp_path_bezier_2d\njp_path_bezier_2d_continual\njp_path_bezier_3d\njp_path_bezier_3d_continual\njp_rectangle_geom\njp_get_bounds\n```",  # noqa
    ##################################################
    "## Mask": "## マスク",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nsvg_mask\n```": "```{toctree}\n:maxdepth: 1\njp_svg_mask\n```",  # noqa
    ##################################################
    "## Common event interfaces": "## イベントの共通の各インターフェイス",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nabout_handler_options_type\nevent_prevent_default_and_stop_propagation\nbind_and_trigger_custom_event\n```": "```{toctree}\n:maxdepth: 1\njp_about_handler_options_type\njp_event_prevent_default_and_stop_propagation\njp_bind_and_trigger_custom_event\n```",  # noqa
    ##################################################
    "## MouseEvent class and mouse event binding": "## MouseEvent クラスとマウスイベントの設定",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nmouse_event_abstract\nmouse_event_basic\nclick\ndblclick\nmousedown_and_mouseup\nmouseover_and_mouseout\nmousemove\n```": "```{toctree}\n:maxdepth: 1\njp_mouse_event_abstract\njp_mouse_event_basic\njp_click\njp_dblclick\njp_mousedown_and_mouseup\njp_mouseover_and_mouseout\njp_mousemove\n```",  # noqa
    ##################################################
    "## Branch instruction": "## 条件分岐の制御",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nif\nelif\nelse\nbranch_instruction_variables_reverting_setting\nreturn\n```": "```{toctree}\n:maxdepth: 1\njp_if\njp_elif\njp_else\njp_branch_instruction_variables_reverting_setting\njp_return\n```",  # noqa
    ##################################################
    "## Loop": "## ループ",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nfor_array_indices\nfor_array_values\nfor_array_indices_and_values\nfor_dict_keys\nfor_dict_values\nfor_dict_keys_and_values\ncontinue\nrange\n```": "```{toctree}\n:maxdepth: 1\njp_for_array_indices\njp_for_array_values\njp_for_array_indices_and_values\njp_for_dict_keys\njp_for_dict_values\njp_for_dict_keys_and_values\njp_continue\njp_range\n```",  # noqa
    ##################################################
    "## Timer and enter-frame": "## タイマーとenter-frame",
    ##################################################
    "```{toctree}\n:maxdepth: 1\ntimer\ntimer_event\ntimer_delay\nfps\ntimer_repeat_count\ntimer_start_and_stop\ntimer_complete\ntimer_reset\nenter_frame\nunbind_enter_frame_and_unbind_enter_frame_all\n```": "```{toctree}\n:maxdepth: 1\njp_timer\njp_timer_event\njp_timer_delay\njp_fps\njp_timer_repeat_count\njp_timer_start_and_stop\njp_timer_complete\njp_timer_reset\njp_enter_frame\njp_unbind_enter_frame_and_unbind_enter_frame_all\n```",  # noqa
    ##################################################
    "## DateTime class": "## DateTime クラス",
    ##################################################
    "```{toctree}\n:maxdepth: 1\ndatetime\ndatetime_year\ndatetime_month\ndatetime_day\ndatetime_hour\ndatetime_minute\ndatetime_second\ndatetime_millisecond\ndatetime_weekday_js_and_weekday_py\ndatetime_now\ndatetime_set_month_end\n```": "```{toctree}\n:maxdepth: 1\njp_datetime\njp_datetime_year\njp_datetime_month\njp_datetime_day\njp_datetime_hour\njp_datetime_minute\njp_datetime_second\njp_datetime_millisecond\njp_datetime_weekday_js_and_weekday_py\njp_datetime_now\njp_datetime_set_month_end\n```",  # noqa
    ##################################################
    "## TimeDelta class": "## TimeDelta クラス",
    ##################################################
    "```{toctree}\n:maxdepth: 1\ntimedelta\ntimedelta_days\ntimedelta_total_seconds\n```": "```{toctree}\n:maxdepth: 1\njp_timedelta\njp_timedelta_days\njp_timedelta_total_seconds\n```",  # noqa
    ##################################################
    "## Animation": "## アニメーション",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nanimation_interfaces_abstract\nanimation_event\nanimation_duration\nanimation_delay\nanimation_return_value\nanimation_base_start\nanimation_complete\nanimation_method_chaining\nanimation_base_target\nanimation_pause_and_play\nanimation_reset\nanimation_finish\nanimation_reverse\nanimation_time\neasing_enum\nsequential_animation\nanimation_parallel\nanimation_move\nanimation_x\nanimation_y\nanimation_width_and_height\nanimation_fill_color\nanimation_fill_alpha\nanimation_line_color\nanimation_line_alpha\nanimation_line_thickness\nanimation_radius\nanimation_rotation_around_center\nanimation_rotation_around_point\nanimation_scale_x_and_y_from_center\nanimation_scale_x_and_y_from_point\n```": "```{toctree}\n:maxdepth: 1\njp_animation_interfaces_abstract\njp_animation_event\njp_animation_duration\njp_animation_delay\njp_animation_return_value\njp_animation_base_start\njp_animation_complete\njp_animation_method_chaining\njp_animation_base_target\njp_animation_pause_and_play\njp_animation_reset\njp_animation_finish\njp_animation_reverse\njp_animation_time\njp_easing_enum\njp_sequential_animation\njp_animation_parallel\njp_animation_move\njp_animation_x\njp_animation_y\njp_animation_width_and_height\njp_animation_fill_color\njp_animation_fill_alpha\njp_animation_line_color\njp_animation_line_alpha\njp_animation_line_thickness\njp_animation_radius\njp_animation_rotation_around_center\njp_animation_rotation_around_point\njp_animation_scale_x_and_y_from_center\njp_animation_scale_x_and_y_from_point\n```",  # noqa
    ##################################################
    "## Mathematics": "## 数学",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nmath_min\nmath_max\nmath_trunc\nmath_clamp\n```": "```{toctree}\n:maxdepth: 1\njp_math_min\njp_math_max\njp_math_trunc\njp_math_clamp\n```",  # noqa
    ##################################################
    "## Other manipulation interfaces": "## その他の操作関係の各インターフェイス",
    ##################################################
    "```{toctree}\n:maxdepth: 1\ndelete\n```": "```{toctree}\n:maxdepth: 1\njp_delete\n```",  # noqa
    ##################################################
    "## Debugging": "## デバッグ",
    ##################################################
    "```{toctree}\n:maxdepth: 1\ntrace\nset_debug_mode\nunset_debug_mode\nadd_debug_info_setting\nvariable_name_suffix\n```": "```{toctree}\n:maxdepth: 1\njp_trace\njp_set_debug_mode\njp_unset_debug_mode\njp_add_debug_info_setting\njp_variable_name_suffix\n```",  # noqa
    ##################################################
    "## Testing": "## テスト",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nassertion_basic_behavior\nassert_equal_and_not_equal\nassert_true_and_false\nassert_greater_and_greater_equal\nassert_less_and_less_equal\nassert_arrays_equal_and_arrays_not_equal\nassert_dicts_equal_and_dicts_not_equal\nassert_defined_and_undefined\n```": "```{toctree}\n:maxdepth: 1\njp_assertion_basic_behavior\njp_assert_equal_and_not_equal\njp_assert_true_and_false\njp_assert_greater_and_greater_equal\njp_assert_less_and_less_equal\njp_assert_arrays_equal_and_arrays_not_equal\njp_assert_dicts_equal_and_dicts_not_equal\njp_assert_defined_and_undefined\n```",  # noqa
    ##################################################
    "## Material design": "## マテリアルデザイン",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nmaterial_icon\n```": "```{toctree}\n:maxdepth: 1\njp_material_icon\n```",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: array_comparison.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Array class comparison interfaces": "# Array クラスの比較の各インターフェイス",
    ##################################################
    "This page explains the `Array` class comparison interfaces (equal and not equal comparison).": "このページでは`Array`クラスにおける等値と非等値の比較の各インターフェイスについて説明します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `Array` value can compare with a Python built-in `list` value and `Array` value. A return value becomes the `Boolean` type.": "`Array`クラスの値はPythonビルトインのリスト及び別の`Array`クラスの値と比較を行うことができます。結果は`Boolean`型となります。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\narr: ap.Array[int] = ap.Array([1, 3, 5])\nresult: ap.Boolean = arr == [1, 3, 5]\nassert result\n```": "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\narr: ap.Array[int] = ap.Array([1, 3, 5])\nresult: ap.Boolean = arr == [1, 3, 5]\nassert result\n```",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\narr: ap.Array[int] = ap.Array([1, 3, 5])\nother_arr: ap.Array[int] = ap.Array([1, 3, 5])\nresult: ap.Boolean = arr == other_arr\nassert result\n```": "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\narr: ap.Array[int] = ap.Array([1, 3, 5])\nother_arr: ap.Array[int] = ap.Array([1, 3, 5])\nresult: ap.Boolean = arr == other_arr\nassert result\n```",  # noqa
    ##################################################
    "The equal comparison operator (`==`) and not equal comparison operator (`!=`) are supported:": "等値の比較記号（`==`）と非等値の比較記号（`!=`）をサポートしています:",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\narr: ap.Array[int] = ap.Array([1, 3, 5])\nresult: ap.Boolean = arr != [2, 4, 6]\nassert result\n```": "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\narr: ap.Array[int] = ap.Array([1, 3, 5])\nresult: ap.Boolean = arr != [2, 4, 6]\nassert result\n```",  # noqa
}



"""The class implementation for the outlined `contact_mail` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialContactMailOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `contact_mail` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M22 3H2C.9 3 0 3.9 0 5v14c0 1.1.9 2 2 2h20c1.1 0 1.99-.9 1.99-2L24 5c0-1.1-.9-2-2-2zm0 16H2V5h20v14zM21 6h-7v5h7V6zm-1 2l-2.5 1.75L15 8V7l2.5 1.75L20 7v1zM9 12c1.65 0 3-1.35 3-3s-1.35-3-3-3-3 1.35-3 3 1.35 3 3 3zm0-4c.55 0 1 .45 1 1s-.45 1-1 1-1-.45-1-1 .45-1 1-1zm6 8.59c0-2.5-3.97-3.58-6-3.58s-6 1.08-6 3.58V18h12v-1.41zM5.48 16c.74-.5 2.22-1 3.52-1s2.77.49 3.52 1H5.48z"/></svg>'  # noqa



"""The SVG mask property-related mix-in class implementation.
"""

from typing import Optional

from apysc._html.debug_mode import add_debug_info_setting
from apysc._mask.svg_mask import SvgMask
from apysc._validation import arg_validation_decos


class SvgMaskMixIn:
    """
    The SVG mask property-related mix-in class.
    """

    _mask: Optional[SvgMask] = None

    @property
    @add_debug_info_setting(module_name=__name__)
    def svg_mask(self) -> Optional[SvgMask]:
        """
        Get an SVG mask setting. If the mask is not set, this property becomes `None`.

        Returns
        -------
        mask : Optional[SvgMask]
            A mask setting.

        References
        ----------
        - SvgMask class and its related interfaces
            - https://simon-ritchie.github.io/apysc/en/svg_mask.html

        Examples
        --------
        >>> import apysc as ap

        >>> _ = ap.Stage()
        >>> mask: ap.SvgMask = ap.SvgMask()
        >>> circle: ap.Circle = ap.Circle(
        ...     x=50, y=50, radius=50, fill_color=ap.Colors.CYAN_00AAFF
        ... )
        >>> mask.add_svg_masking_object(masking_object=circle)
        >>> rectangle: ap.Rectangle = ap.Rectangle(
        ...     x=0, y=0, width=100, height=100, fill_color=ap.Colors.CYAN_00AAFF
        ... )
        >>> rectangle.svg_mask = mask
        >>> assert rectangle.svg_mask == mask
        """
        return self._mask

    @svg_mask.setter
    @add_debug_info_setting(module_name=__name__)
    @arg_validation_decos.is_svg_mask(arg_position_index=1, optional=True)
    def svg_mask(self, value: Optional[SvgMask]) -> None:
        """
        Set an SVG mask setting.

        Parameters
        ----------
        value : Optional[SvgMask]
            SVG mask setting to set.

        References
        ----------
        - SvgMask class and its related interfaces
            - https://simon-ritchie.github.io/apysc/en/svg_mask.html

        Examples
        --------
        >>> import apysc as ap

        >>> _ = ap.Stage()
        >>> mask: ap.SvgMask = ap.SvgMask()
        >>> circle: ap.Circle = ap.Circle(
        ...     x=50, y=50, radius=50, fill_color=ap.Colors.CYAN_00AAFF
        ... )
        >>> mask.add_svg_masking_object(masking_object=circle)
        >>> rectangle: ap.Rectangle = ap.Rectangle(
        ...     x=0, y=0, width=100, height=100, fill_color=ap.Colors.CYAN_00AAFF
        ... )
        >>> rectangle.svg_mask = mask
        >>> assert rectangle.svg_mask == mask
        """
        from apysc._expression import expression_data_util
        from apysc._validation.variable_name_validation import (
            validate_variable_name_mixin_type,
        )

        self_variable_name: str = validate_variable_name_mixin_type(
            instance=self
        ).variable_name

        if value is None:
            expression: str = f"{self_variable_name}.unclip();"
        else:
            expression = f"{self_variable_name}.clipWith({value.variable_name});"

        expression_data_util.append_js_expression(expression=expression)
        self._mask = value



"""Class implementation for copy interface.
"""

from copy import deepcopy
from typing import Generic
from typing import TypeVar
from typing import cast

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.copy_interface import CopyInterface
from apysc._type.type_name_mixin import TypeNameMixIn
from apysc._type.variable_name_mixin import VariableNameMixIn

_SelfType = TypeVar("_SelfType", bound="CopyMixIn")


class CopyMixIn(
    TypeNameMixIn,
    VariableNameMixIn,
    CopyInterface,
    Generic[_SelfType],
):
    @final
    @add_debug_info_setting(module_name=__name__)
    def copy(self) -> _SelfType:
        """
        Make a deep copy of this instance.

        Returns
        -------
        result : _SelfType
            Copied instance.
        """
        from apysc._expression import expression_variables_util
        from apysc._expression.event_handler_scope import TemporaryNotHandlerScope

        with TemporaryNotHandlerScope():
            result: _SelfType = cast(_SelfType, deepcopy(self))
            result.variable_name = expression_variables_util.get_next_variable_name(
                type_name=self.type_name
            )
            self._append_copy_expression(result_variable_name=result.variable_name)
        self._append_value_updating_cpy_exp_to_handler_scope(
            result_variable_name=result.variable_name
        )
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_value_updating_cpy_exp_to_handler_scope(
        self, *, result_variable_name: str
    ) -> None:
        """
        Append a value updating copy expression
        if the current scope is an event handler's one.

        Parameters
        ----------
        result_variable_name : str
            Copied value's variable name.
        """
        from apysc._expression import event_handler_scope
        from apysc._expression import expression_data_util
        from apysc._type.type_util import is_immutable_type

        if not event_handler_scope.current_scope_is_in_event_handler():
            return
        if is_immutable_type(value=self):
            expression: str = f"{result_variable_name} = {self.variable_name};"
        else:
            expression = f"{result_variable_name} = " f"cpy({self.variable_name});"
        expression_data_util.append_js_expression(expression=expression)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_copy_expression(self, *, result_variable_name: str) -> None:
        """
        Append copy expression.

        Parameters
        ----------
        result_variable_name : str
            Copied value's variable name.
        """
        from apysc._expression import expression_data_util
        from apysc._type.type_util import is_immutable_type

        if is_immutable_type(value=self):
            expression: str = f"var {result_variable_name} = {self.variable_name};"
        else:
            expression = f"var {result_variable_name} = cpy({self.variable_name});"
        expression_data_util.append_js_expression(expression=expression)



"""Class implementation of the floating-point number value.
"""

from typing import Any
from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._loop.initialize_with_base_value_interface import (
    InitializeWithBaseValueInterface,
)
from apysc._type.hashable_interface import HashableInterface
from apysc._type.number_value_mixin import NumberValueMixIn
from apysc._type.to_fixed_mixin import ToFixedMixIn
from apysc._type.to_hex_mixin import ToHexMixIn
from apysc._type.to_string_mixin import ToStringMixIn
from apysc._validation import arg_validation_decos


class Number(
    NumberValueMixIn[float, "Number"],
    InitializeWithBaseValueInterface,
    ToStringMixIn,
    ToFixedMixIn,
    HashableInterface,
    ToHexMixIn,
):
    """
    Floating point number class for the apysc library.

    Notes
    -----
    The `Float` class is the alias of the Number,
    and it behaves the same as the Number class.

    References
    ----------
    - Int and Number
        - https://simon-ritchie.github.io/apysc/en/int_and_number.html
    - Int and Number common arithmetic operations
        - https://simon-ritchie.github.io/apysc/en/int_and_number_arithmetic_operations.html  # noqa
    - Int and Number common comparison operations
        - https://simon-ritchie.github.io/apysc/en/int_and_number_comparison_operations.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> _ = ap.Stage()
    >>> number: ap.Number = ap.Number(10.5)
    >>> number
    Number(10.5)

    >>> number == 10.5
    Boolean(True)

    >>> number == ap.Number(10.5)
    Boolean(True)

    >>> number >= 10.5
    Boolean(True)

    >>> number += 10.3
    >>> number
    Number(20.8)
    """

    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        value: Union[int, float, NumberValueMixIn],
        *,
        variable_name_suffix: str = "",
        skip_init_substitution_expression_appending: bool = False,
    ) -> None:
        """
        Floating point number class for apysc library.

        Parameters
        ----------
        value : int or float or Int or Number
            Initial floating point number value. This class
            casts it to float if you specify int or Int value.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        skip_init_substitution_expression_appending : bool, default False
            A boolean indicates whether to skip an initial substitution
            expression or not. This class uses this option internally.

        Notes
        -----
        The `Float` class is the alias of the Number, and it behaves
        the same as the Number class.

        References
        ----------
        - Int and Number
            - https://simon-ritchie.github.io/apysc/en/int_and_number.html
        - Int and Number common arithmetic operations
            - https://simon-ritchie.github.io/apysc/en/int_and_number_arithmetic_operations.html  # noqa
        - Int and Number common comparison operations
            - https://simon-ritchie.github.io/apysc/en/int_and_number_comparison_operations.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> number: ap.Number = ap.Number(10.5)
        >>> number
        Number(10.5)

        >>> number == 10.5
        Boolean(True)

        >>> number == ap.Number(10.5)
        Boolean(True)

        >>> number >= 10.5
        Boolean(True)

        >>> number += 10.3
        >>> number
        Number(20.8)
        """
        from apysc._converter import cast
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        from apysc._expression.event_handler_scope import TemporaryNotHandlerScope

        with TemporaryNotHandlerScope():
            TYPE_NAME: str = var_names.NUMBER
            self.variable_name = expression_variables_util.get_next_variable_name(
                type_name=TYPE_NAME
            )
            super(Number, self).__init__(
                value=value,
                type_name=TYPE_NAME,
                variable_name_suffix=variable_name_suffix,
            )
            self._value = cast.to_float_from_int(int_or_float=self.value)
            self._append_constructor_expression()

        self._append_initial_substitution_expression_if_in_handler_scope(
            skip_appending=skip_init_substitution_expression_appending,
        )

    @final
    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    def _set_value_and_skip_expression_appending(
        self, *, value: Union[int, float, Any]
    ) -> None:
        """
        Update value attribute and skip expression appending.

        Parameters
        ----------
        value : int or float or Int or Number
            Any number value to set. If a float or Number
            value is specified, this interface casts its
            value to an integer.
        """
        from apysc._converter import cast

        if isinstance(value, NumberValueMixIn):
            value._value = cast.to_float_from_int(int_or_float=value._value)
            value_ = value._value
        else:
            value = cast.to_float_from_int(int_or_float=value)
            value_ = value
        self._value = value_

    def __repr__(self) -> str:
        """
        Get a representation string of this instance.

        Returns
        -------
        repr_str : str
            Representation string of this instance.
        """
        if not hasattr(self, "_value"):
            repr_str: str = "Number(0)"
        else:
            repr_str = f"Number({self._value})"
        return repr_str

    @classmethod
    @final
    def _initialize_with_base_value(cls) -> "Number":
        """
        Initialize this class with a base value(s).

        Returns
        -------
        num_value : Number
            An initialized number value.
        """
        return Number(0)

    @final
    def __hash__(self) -> int:
        """
        Get a hashed value.

        Returns
        -------
        hashed_value : int
            A hashed value.
        """
        return hash(self._value)



"""The class implementation for the outlined `thumbs_up_down` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialThumbsUpDownOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `thumbs_up_down` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0zm0 0h24v24H0V0z" fill="none"/><path d="M12 6c0-.55-.45-1-1-1H5.82l.66-3.18.02-.23c0-.31-.13-.59-.33-.8L5.38 0 .44 4.94C.17 5.21 0 5.59 0 6v6.5c0 .83.67 1.5 1.5 1.5h6.75c.62 0 1.15-.38 1.38-.91l2.26-5.29c.07-.17.11-.36.11-.55V6zm-2 1.13L7.92 12H2V6.21l1.93-1.93L3.36 7H10v.13zM22.5 10h-6.75c-.62 0-1.15.38-1.38.91l-2.26 5.29c-.07.17-.11.36-.11.55V18c0 .55.45 1 1 1h5.18l-.66 3.18-.02.24c0 .31.13.59.33.8l.79.78 4.94-4.94c.27-.27.44-.65.44-1.06v-6.5c0-.83-.67-1.5-1.5-1.5zm-.5 7.79l-1.93 1.93.57-2.72H14v-.13L16.08 12H22v5.79z"/></svg>'  # noqa



"""The class implementation for the `fiber_smart_record` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialFiberSmartRecordIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `fiber_smart_record` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M24 24H0V0h24v24z" fill="none"/><g><circle cx="9" cy="12" r="8"/><path d="M17 4.26v2.09c2.33.82 4 3.04 4 5.65s-1.67 4.83-4 5.65v2.09c3.45-.89 6-4.01 6-7.74s-2.55-6.85-6-7.74z"/></g></svg>'  # noqa



"""The class implementation for the `perm_camera_mic` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialPermCameraMicIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `perm_camera_mic` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M20 5h-3.17L15 3H9L7.17 5H4c-1.1 0-2 .9-2 2v12c0 1.1.9 2 2 2h7v-2.09c-2.83-.48-5-2.94-5-5.91h2c0 2.21 1.79 4 4 4s4-1.79 4-4h2c0 2.97-2.17 5.43-5 5.91V21h7c1.1 0 2-.9 2-2V7c0-1.1-.9-2-2-2zm-6 8c0 1.1-.9 2-2 2s-2-.9-2-2V9c0-1.1.9-2 2-2s2 .9 2 2v4z"/></svg>'  # noqa



"""The mix-in class implementation for the material design button's label.
"""

from typing import List
from typing import Optional
from typing import Union

from apysc._color.color import Color
from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase
from apysc._display.svg_text import SvgText
from apysc._geom.rectangle_geom import RectangleGeom
from apysc._type.array import Array
from apysc._type.int import Int
from apysc._type.string import String


class MaterialButtonLabelMixIn:
    _label: String
    _label_text: SvgText
    _text_color: Color
    _font_family: Array[String]
    _font_size: Int

    def _initialize_label(
        self,
        *,
        label: Union[str, String],
        text_color: Color,
        font_family: Optional[Union[Array[String], List[str]]],
        font_size: Union[int, Int],
    ) -> None:
        """
        Initialize the label setting with the specified argument.

        Parameters
        ----------
        label : Union[str, String]
            A label text to display on this button.
        text_color : Color
            The color of the label text.
        font_family : Optional[Union[Array[String], List[str]]]
            A font-family setting.
        font_size : Union[int, Int]
            A font-size setting.
        """
        from apysc._converter.list_of_strs_to_array import (
            list_of_strs_to_array_of_string,
        )
        from apysc._converter.to_apysc_val_from_builtin import (
            get_copied_int_from_builtin_val,
        )
        from apysc._converter.to_apysc_val_from_builtin import (
            get_copied_string_from_builtin_val,
        )
        from apysc._display.child_mixin import ChildMixIn
        from apysc._material_design.setting.material_settings_utils import (
            MaterialSettingsUtils,
        )
        from apysc._type.variable_name_suffix_utils import (
            get_attr_or_variable_name_suffix,
        )
        from apysc._validation.display_validation import (
            validate_display_object_container,
        )

        self._label = get_copied_string_from_builtin_val(
            string=label,
            variable_name_suffix=get_attr_or_variable_name_suffix(
                instance=self, value_identifier="material_button_label"
            ),
        )
        self._text_color = text_color
        self._font_family = MaterialSettingsUtils.get_font_family(
            argument_font_family=list_of_strs_to_array_of_string(
                optional_list_or_arr=font_family,
                variable_name_suffix=get_attr_or_variable_name_suffix(
                    instance=self, value_identifier="material_button_label_font_family"
                ),
            )
        )
        self._font_size = get_copied_int_from_builtin_val(
            integer=font_size,
            variable_name_suffix=get_attr_or_variable_name_suffix(
                instance=self, value_identifier="material_button_label_font_size"
            ),
        )

        parent: ChildMixIn = validate_display_object_container(container_object=self)
        self._label_text = SvgText(
            text=self._label,
            font_size=self._font_size,
            font_family=self._font_family,
            fill_color=self._text_color,
            variable_name_suffix=get_attr_or_variable_name_suffix(
                instance=self, value_identifier="material_button_label_text"
            ),
            parent=parent,
        )

    def _locate_label_text(
        self,
        *,
        label_text_bounding_box: RectangleGeom,
        prefix_icon: Optional[FixedHtmlSvgIconBase],
    ) -> None:
        """
        Locate the label text.

        Parameters
        ----------
        label_text_bounding_box : RectangleGeom
            The bounding box of the label text.
        prefix_icon : Optional[FixedHtmlSvgIconBase]
            An icon to display on the left side of the label.
        """
        from apysc._material_design.button.material_button_const import BUTTON_HEIGHT
        from apysc._material_design.button.material_button_const import (
            ICON_INNER_PADDING_WIDTH,
        )
        from apysc._material_design.button.material_button_const import (
            ICON_OUTER_PADDING_WIDTH,
        )
        from apysc._material_design.button.material_button_const import ICON_SIZE
        from apysc._material_design.button.material_button_const import (
            NO_ICON_OUTER_PADDING,
        )
        from apysc._type.number import Number
        from apysc._type.variable_name_suffix_utils import (
            get_attr_or_variable_name_suffix,
        )

        suffix: str = get_attr_or_variable_name_suffix(
            instance=self,
            value_identifier="material_button_label_text_x",
        )
        if prefix_icon is None:
            self._label_text.x = Number(
                NO_ICON_OUTER_PADDING,
                variable_name_suffix=suffix,
            )
        else:
            self._label_text.x = Number(
                ICON_OUTER_PADDING_WIDTH + ICON_SIZE + ICON_INNER_PADDING_WIDTH,
                variable_name_suffix=suffix,
            )

        half_button_height: float = BUTTON_HEIGHT / 2
        suffix = get_attr_or_variable_name_suffix(
            instance=self,
            value_identifier="material_button_label_text_y",
        )
        self._label_text.y = (
            Number(
                half_button_height,
                variable_name_suffix=suffix,
            )
            - label_text_bounding_box.height / 2
        )



"""The `MaterialColorScheme` class implementation.
"""

from typing import Optional

from apysc._color.color import Color
from apysc._html.debug_mode import add_debug_info_setting
from apysc._validation import arg_validation_decos


class MaterialColorScheme:
    """
    The color scheme class for the material design.

    Notes
    -----
    The apysc library refers to the Flutter's ColorScheme class.

    References
    ----------
    - ColorScheme class (Flutter)
        - https://api.flutter.dev/flutter/material/ColorScheme-class.html
    - Flutter license (BSD 3-Clause "New" or "Revised" License)
        - https://github.com/flutter/flutter/blob/master/LICENSE
    """

    _primary: Color
    _on_primary: Color
    _secondary: Color
    _on_secondary: Color
    _error: Color
    _on_error: Color
    _surface: Color
    _on_surface: Color
    _primary_container: Optional[Color] = None
    _on_primary_container: Optional[Color] = None
    _secondary_container: Optional[Color] = None
    _on_secondary_container: Optional[Color] = None
    _tertiary: Optional[Color] = None
    _on_tertiary: Optional[Color] = None
    _tertiary_container: Optional[Color] = None
    _on_tertiary_container: Optional[Color] = None
    _error_container: Optional[Color] = None
    _on_error_container: Optional[Color] = None
    _outline: Optional[Color] = None
    _outline_variant: Optional[Color] = None

    # primary
    @arg_validation_decos.is_color(arg_position_index=1, optional=False)
    # on_primary
    @arg_validation_decos.is_color(arg_position_index=2, optional=False)
    # secondary
    @arg_validation_decos.is_color(arg_position_index=3, optional=False)
    # on_secondary
    @arg_validation_decos.is_color(arg_position_index=4, optional=False)
    # error
    @arg_validation_decos.is_color(arg_position_index=5, optional=False)
    # on_error
    @arg_validation_decos.is_color(arg_position_index=6, optional=False)
    # surface
    @arg_validation_decos.is_color(arg_position_index=7, optional=False)
    # on_surface
    @arg_validation_decos.is_color(arg_position_index=8, optional=False)
    # primary_container
    @arg_validation_decos.is_color(arg_position_index=9, optional=True)
    # on_primary_container
    @arg_validation_decos.is_color(arg_position_index=10, optional=True)
    # secondary_container
    @arg_validation_decos.is_color(arg_position_index=11, optional=True)
    # on_secondary_container
    @arg_validation_decos.is_color(arg_position_index=12, optional=True)
    # tertiary
    @arg_validation_decos.is_color(arg_position_index=13, optional=True)
    # on_tertiary
    @arg_validation_decos.is_color(arg_position_index=14, optional=True)
    # tertiary_container
    @arg_validation_decos.is_color(arg_position_index=15, optional=True)
    # on_tertiary_container
    @arg_validation_decos.is_color(arg_position_index=16, optional=True)
    # error_container
    @arg_validation_decos.is_color(arg_position_index=17, optional=True)
    # on_error_container
    @arg_validation_decos.is_color(arg_position_index=18, optional=True)
    # outline
    @arg_validation_decos.is_color(arg_position_index=19, optional=True)
    # outline_variant
    @arg_validation_decos.is_color(arg_position_index=20, optional=True)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        *,
        primary: Color,
        on_primary: Color,
        secondary: Color,
        on_secondary: Color,
        error: Color,
        on_error: Color,
        surface: Color,
        on_surface: Color,
        primary_container: Optional[Color] = None,
        on_primary_container: Optional[Color] = None,
        secondary_container: Optional[Color] = None,
        on_secondary_container: Optional[Color] = None,
        tertiary: Optional[Color] = None,
        on_tertiary: Optional[Color] = None,
        tertiary_container: Optional[Color] = None,
        on_tertiary_container: Optional[Color] = None,
        error_container: Optional[Color] = None,
        on_error_container: Optional[Color] = None,
        outline: Optional[Color] = None,
        outline_variant: Optional[Color] = None,
    ) -> None:
        """
        The color scheme class for the material design.

        Parameters
        ----------
        primary : Color
            The main color displayed most frequently.
        on_primary : Color
            A color to use on a primary color widget.
        secondary : Color
            An accent color used for less prominent widgets than the primary color.
        on_secondary : Color
            A color to use on a secondary color widget.
        error : Color
            A color to use for input validation errors.
        on_error : Color
            A color to use on an error color widget.
        surface : Color
            The background color for widgets like the material design's card.
        on_surface : Color
            A color to use on a surface color widget.
        primary_container : Optional[Color], optional
            A color to use on a primary color container widget or
            when you want to use this in a less prominent way
            than the primary color.
        on_primary_container : Optional[Color], optional
            A color to use on a primary_container color widget.
        secondary_container : Optional[Color], optional
            A color to use on a secondary color container widget or
            when you want to use this in a less prominent way
            than the secondary color.
        on_secondary_container : Optional[Color], optional
            A color to use on a secondary_container color widget.
        tertiary : Optional[Color], optional
            An accent color following the primary and secondary colors.
        on_tertiary : Optional[Color], optional
            A color to use on a tertiary color widget.
        tertiary_container : Optional[Color], optional
            A color to use on a tertiary color container widget or
            when you want to use this in a less prominent way
            than the tertiary color.
        on_tertiary_container : Optional[Color], optional
            A color to use on a tertiary_container color widget.
        error_container : Optional[Color], optional
            A color to use on an error color container widget or
            when you want to use this in a less prominent way
            than the error color.
        on_error_container : Optional[Color], optional
            A color to use on an error_container color widget.
        outline : Optional[Color], optional
            A widget outline color.
        outline_variant : Optional[Color], optional
            A color for an widget's outline that is less prominent than
            the outline color.

        Notes
        -----
        The apysc library refers to the Flutter's ColorScheme class.

        References
        ----------
        - ColorScheme class (Flutter)
            - https://api.flutter.dev/flutter/material/ColorScheme-class.html
        - Flutter license (BSD 3-Clause "New" or "Revised" License)
            - https://github.com/flutter/flutter/blob/master/LICENSE
        """
        self._primary = primary.copy()
        self._on_primary = on_primary.copy()
        self._secondary = secondary.copy()
        self._on_secondary = on_secondary.copy()
        self._error = error.copy()
        self._on_error = on_error.copy()
        self._surface = surface.copy()
        self._on_surface = on_surface.copy()
        if primary_container is not None:
            self._primary_container = primary_container.copy()
        if on_primary_container is not None:
            self._on_primary_container = on_primary_container.copy()
        if secondary_container is not None:
            self._secondary_container = secondary_container.copy()
        if on_secondary_container is not None:
            self._on_secondary_container = on_secondary_container.copy()
        if tertiary is not None:
            self._tertiary = tertiary.copy()
        if on_tertiary is not None:
            self._on_tertiary = on_tertiary.copy()
        if tertiary_container is not None:
            self._tertiary_container = tertiary_container.copy()
        if on_tertiary_container is not None:
            self._on_tertiary_container = on_tertiary_container.copy()
        if error_container is not None:
            self._error_container = error_container.copy()
        if on_error_container is not None:
            self._on_error_container = on_error_container.copy()
        if outline is not None:
            self._outline = outline.copy()
        if outline_variant is not None:
            self._outline_variant = outline_variant.copy()

    @property
    def primary(self) -> Color:
        """
        Get the `primary` color.

        Returns
        -------
        primary : Color
            The main color displayed most frequently.
        """
        return self._primary.copy()

    @property
    def on_primary(self) -> Color:
        """
        Get the `on_primary` color.

        Returns
        -------
        on_primary : Color
            A color to use on a primary color widget.
        """
        return self._on_primary.copy()

    @property
    def secondary(self) -> Color:
        """
        Get the `secondary` color.

        Returns
        -------
        secondary : Color
            An accent color used for less prominent widgets than the primary color.
        """
        return self._secondary.copy()

    @property
    def on_secondary(self) -> Color:
        """
        Get the `on_secondary` color.

        Returns
        -------
        on_secondary : Color
            A color to use on a secondary color widget.
        """
        return self._on_secondary.copy()

    @property
    def error(self) -> Color:
        """
        Get the `error` color.

        Returns
        -------
        error : Color
            A color to use for input validation errors.
        """
        return self._error.copy()

    @property
    def on_error(self) -> Color:
        """
        Get the `on_error` color.

        Returns
        -------
        on_error : Color
            A color to use on an error color widget.
        """
        return self._on_error.copy()

    @property
    def surface(self) -> Color:
        """
        Get the `surface` color.

        Returns
        -------
        surface : Color
            The background color for widgets like the material design's card.
        """
        return self._surface.copy()

    @property
    def on_surface(self) -> Color:
        """
        Get the `on_surface` color.

        Returns
        -------
        on_surface : Color
            A color to use on a surface color widget.
        """
        return self._on_surface.copy()

    @property
    def primary_container(self) -> Optional[Color]:
        """
        Get the `primary_container` color.

        Returns
        -------
        primary_container : Optional[Color]
            A color to use on a primary color container widget or
            when you want to use this in a less prominent way
            than the primary color.
        """
        if self._primary_container is None:
            return None
        return self._primary_container.copy()

    @property
    def on_primary_container(self) -> Optional[Color]:
        """
        Get the `on_primary_container` color.

        Returns
        -------
        on_primary_container : Optional[Color]
            A color to use on a primary_container color widget.
        """
        if self._on_primary_container is None:
            return None
        return self._on_primary_container.copy()

    @property
    def secondary_container(self) -> Optional[Color]:
        """
        Get the `secondary_container` color.

        Returns
        -------
        secondary_container : Optional[Color]
            A color to use on a secondary color container widget or
            when you want to use this in a less prominent way
            than the secondary color.
        """
        if self._secondary_container is None:
            return None
        return self._secondary_container.copy()

    @property
    def on_secondary_container(self) -> Optional[Color]:
        """
        Get the `on_secondary_container` color.

        Returns
        -------
        on_secondary_container : Optional[Color]
            A color to use on a secondary_container color widget.
        """
        if self._on_secondary_container is None:
            return None
        return self._on_secondary_container.copy()

    @property
    def tertiary(self) -> Optional[Color]:
        """
        Get the `tertiary` color.

        Returns
        -------
        tertiary : Optional[Color]
            An accent color following the primary and secondary colors.
        """
        if self._tertiary is None:
            return None
        return self._tertiary.copy()

    @property
    def on_tertiary(self) -> Optional[Color]:
        """
        Get the `on_tertiary` color.

        Returns
        -------
        on_tertiary : Optional[Color]
            A color to use on a tertiary color widget.
        """
        if self._on_tertiary is None:
            return None
        return self._on_tertiary.copy()

    @property
    def tertiary_container(self) -> Optional[Color]:
        """
        Get the `tertiary_container` color.

        Returns
        -------
        tertiary_container : Optional[Color]
            A color to use on a tertiary color container widget or
            when you want to use this in a less prominent way
            than the tertiary color.
        """
        if self._tertiary_container is None:
            return None
        return self._tertiary_container.copy()

    @property
    def on_tertiary_container(self) -> Optional[Color]:
        """
        Get the `on_tertiary_container` color.

        Returns
        -------
        on_tertiary_container : Optional[Color]
            A color to use on a tertiary_container color widget.
        """
        if self._on_tertiary_container is None:
            return None
        return self._on_tertiary_container.copy()

    @property
    def error_container(self) -> Optional[Color]:
        """
        Get the `error_container` color.

        Returns
        -------
        error_container : Optional[Color]
            A color to use on an error color container widget or
            when you want to use this in a less prominent way
            than the error color.
        """
        if self._error_container is None:
            return None
        return self._error_container.copy()

    @property
    def on_error_container(self) -> Optional[Color]:
        """
        Get the `on_error_container` color.

        Returns
        -------
        on_error_container : Optional[Color]
            A color to use on an error_container color widget.
        """
        if self._on_error_container is None:
            return None
        return self._on_error_container.copy()

    @property
    def outline(self) -> Optional[Color]:
        """
        Get the `outline` color.

        Returns
        -------
        outline : Optional[Color]
            A widget outline color.
        """
        if self._outline is None:
            return None
        return self._outline.copy()

    @property
    def outline_variant(self) -> Optional[Color]:
        """
        Get the `outline_variant` color.

        Returns
        -------
        outline_variant : Optional[Color]
            A color for an widget's outline that is less prominent than
            the outline color.
        """
        if self._outline_variant is None:
            return None
        return self._outline_variant.copy()



"""Class implementations for the month-end related mix-in.
"""

import calendar

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.variable_name_mixin import VariableNameMixIn


class MonthEndMixin(VariableNameMixIn):
    _year: Int
    _month: Int
    _day: Int

    @final
    @add_debug_info_setting(module_name=__name__)
    def set_month_end(self) -> None:
        """
        Set a month-end day.

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> datetime_: ap.DateTime = ap.DateTime(year=2022, month=12, day=5)
        >>> datetime_.set_month_end()
        >>> datetime_.day
        Int(31)

        References
        ----------
        - DateTime class set_month_end interface
            - https://simon-ritchie.github.io/apysc/en/datetime_set_month_end.html
        """
        from apysc._expression import expression_data_util

        variable_name: str = self.variable_name
        expression: str = (
            f"{variable_name}.setDate(1);"
            f"\n{variable_name}.setMonth({variable_name}.getMonth() + 1);"
            f"\n{variable_name}.setDate(0);"
        )
        expression_data_util.append_js_expression(expression=expression)
        month_end_day: int = calendar.monthrange(
            year=self._year._value,
            month=self._month._value,
        )[1]
        self._day._value = month_end_day



"""The class implementation for the outlined `assignment_ind` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAssignmentIndOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `assignment_ind` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M19 3h-4.18C14.4 1.84 13.3 1 12 1s-2.4.84-2.82 2H5c-1.1 0-2 .9-2 2v14c0 1.1.9 2 2 2h14c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2zm-7-.25c.22 0 .41.1.55.25.12.13.2.31.2.5 0 .41-.34.75-.75.75s-.75-.34-.75-.75c0-.19.08-.37.2-.5.14-.15.33-.25.55-.25zM19 19H5V5h14v14zM12 6c-1.65 0-3 1.35-3 3s1.35 3 3 3 3-1.35 3-3-1.35-3-3-3zm0 4c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm-6 6.47V18h12v-1.53c0-2.5-3.97-3.58-6-3.58s-6 1.07-6 3.58zM8.31 16c.69-.56 2.38-1.12 3.69-1.12s3.01.56 3.69 1.12H8.31z"/></svg>'  # noqa



"""Enum definition for the x-axis label position.
"""

from enum import Enum


class XAxisLabelPosition(Enum):
    OUTER_LEFT = "outer_left"
    OUTER_CENTER = "outer_center"
    OUTER_RIGHT = "outer_right"
    INNER_LEFT = "inner_left"
    INNER_CENTER = "inner_center"
    INNER_RIGHT = "inner_right"



"""The class implementation for the `lock_outline` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialLockOutlineIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `lock_outline` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24" x="0"/></g><g><g><g><path d="M12,17c1.1,0,2-0.9,2-2s-0.9-2-2-2s-2,0.9-2,2S10.9,17,12,17z M18,8h-1V6c0-2.76-2.24-5-5-5S7,3.24,7,6v2H6 c-1.1,0-2,0.9-2,2v10c0,1.1,0.9,2,2,2h12c1.1,0,2-0.9,2-2V10C20,8.9,19.1,8,18,8z M8.9,6c0-1.71,1.39-3.1,3.1-3.1 s3.1,1.39,3.1,3.1v2H8.9V6z M18,20H6V10h12V20z"/></g></g></g></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: text_fill_alpha.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Text fill_alpha property": "# テキストの fill_alpha 属性",
    ##################################################
    "This page explains the text-related `fill_alpha` property.": "このページではテキスト関係の`fill_alpha`属性について説明します。",  # noqa
    ##################################################
    "## What property is this?": "## 属性の概要",
    ##################################################
    "The `fill_alpha` property interface updates or gets the instance's fill alpha (opacity).": "fill_alpha`属性のインターフェイスではインスタンスの塗りの透明度更新や取得を行うことができます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The getter or setter interface value becomes (or requires) the `Number` value (0.0 to 1.0).": "getterとsetterの両方のインターフェイスの値は`Number`型の0.0～1.0の範囲の値となります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=350,\n    stage_height=170,\n    stage_elem_id="stage",\n)\nmulti_line_text: ap.MultiLineText = ap.MultiLineText(\n    text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n    "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n    "Ut enim ad minim veniam",\n    width=300,\n    font_size=16,\n    fill_color=ap.Color("#00aaff"),\n    x=25,\n    y=25,\n)\nmulti_line_text.fill_alpha = ap.Number(0.5)\nap.assert_equal(multi_line_text.fill_alpha, 0.5)\n\nap.save_overall_html(dest_dir_path="./text_fill_alpha_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=350,\n    stage_height=170,\n    stage_elem_id="stage",\n)\nmulti_line_text: ap.MultiLineText = ap.MultiLineText(\n    text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n    "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n    "Ut enim ad minim veniam",\n    width=300,\n    font_size=16,\n    fill_color=ap.Color("#00aaff"),\n    x=25,\n    y=25,\n)\nmulti_line_text.fill_alpha = ap.Number(0.5)\nap.assert_equal(multi_line_text.fill_alpha, 0.5)\n\nap.save_overall_html(dest_dir_path="./text_fill_alpha_basic_usage/")\n```',  # noqa
    ##################################################
    "## fill_alpha property API": "## fill_alpha 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a fill-alpha (opacity) value.<hr>": "塗りの透明度の値を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `alpha`: Number": "- `alpha`: Number",
    ##################################################
    "  - A fill-alpha (opacity) value.": "  - 塗りの透明度の値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage(\n...     background_color=ap.Color("#333"),\n...     stage_width=350,\n...     stage_height=170,\n...     stage_elem_id="stage",\n... )\n>>> multi_line_text: ap.MultiLineText = ap.MultiLineText(\n...     text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n...     "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n...     "Ut enim ad minim veniam",\n...     width=300,\n...     font_size=16,\n...     fill_color=ap.Color("#00aaff"),\n...     x=25,\n...     y=25,\n... )\n>>> multi_line_text.fill_alpha = ap.Number(0.5)\n>>> multi_line_text.fill_alpha\nNumber(0.5)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage(\n...     background_color=ap.Color("#333"),\n...     stage_width=350,\n...     stage_height=170,\n...     stage_elem_id="stage",\n... )\n>>> multi_line_text: ap.MultiLineText = ap.MultiLineText(\n...     text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n...     "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n...     "Ut enim ad minim veniam",\n...     width=300,\n...     font_size=16,\n...     fill_color=ap.Color("#00aaff"),\n...     x=25,\n...     y=25,\n... )\n>>> multi_line_text.fill_alpha = ap.Number(0.5)\n>>> multi_line_text.fill_alpha\nNumber(0.5)\n```',  # noqa
}



"""The class implementation for the outlined `track_changes` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialTrackChangesOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `track_changes` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M19.07 4.93l-1.41 1.41C19.1 7.79 20 9.79 20 12c0 4.42-3.58 8-8 8s-8-3.58-8-8c0-4.08 3.05-7.44 7-7.93v2.02C8.16 6.57 6 9.03 6 12c0 3.31 2.69 6 6 6s6-2.69 6-6c0-1.66-.67-3.16-1.76-4.24l-1.41 1.41C15.55 9.9 16 10.9 16 12c0 2.21-1.79 4-4 4s-4-1.79-4-4c0-1.86 1.28-3.41 3-3.86v2.14c-.6.35-1 .98-1 1.72 0 1.1.9 2 2 2s2-.9 2-2c0-.74-.4-1.38-1-1.72V2h-1C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10c0-2.76-1.12-5.26-2.93-7.07z"/></svg>'  # noqa



"""The implementation for the `range` function.
"""

from typing import Any

from apysc._type.array import Array
from apysc._type.int import Int
from apysc._validation import arg_validation_decos


@arg_validation_decos.variadic_args_len_is_between(min_length=1, max_length=3)
@arg_validation_decos.all_variadic_args_are_integers(optional=False)
def range(*args: Any) -> Array[Int]:
    """
    Create a range array of integers.

    Returns
    -------
    arr : Array[Int]
        A created array.

    References
    ----------
    - range function
        - https://simon-ritchie.github.io/apysc/en/range.html

    Examples
    --------
    >>> import apysc as ap

    >>> range_arr: ap.Array[ap.Int] = ap.range(5)
    >>> ap.assert_equal(range_arr, [0, 1, 2, 3, 4])

    >>> range_arr = ap.range(2, 4)
    >>> ap.assert_equal(range_arr, [2, 3])

    >>> range_arr = ap.range(2, 10, 2)
    >>> ap.assert_equal(range_arr, [2, 4, 6, 8])
    """
    from apysc._converter import to_apysc_val_from_builtin

    if len(args) == 1:
        end: Int = to_apysc_val_from_builtin.get_copied_int_from_builtin_val(
            integer=args[0]
        )
        arr: Array[Int] = _create_single_arg_case_arr(end=end)
        return arr

    if len(args) == 2:
        start: Int = to_apysc_val_from_builtin.get_copied_int_from_builtin_val(
            integer=args[0]
        )
        end = to_apysc_val_from_builtin.get_copied_int_from_builtin_val(integer=args[1])
        arr = _create_double_args_case_arr(start=start, end=end)
        return arr

    start = to_apysc_val_from_builtin.get_copied_int_from_builtin_val(integer=args[0])
    end = to_apysc_val_from_builtin.get_copied_int_from_builtin_val(integer=args[1])
    step: Int = to_apysc_val_from_builtin.get_copied_int_from_builtin_val(
        integer=args[2]
    )
    arr = _create_triple_args_case_arr(start=start, end=end, step=step)
    return arr


def _create_triple_args_case_arr(*, start: Int, end: Int, step: Int) -> Array[Int]:
    """
    Create a triple arguments case array.

    Parameters
    ----------
    start : Int
        A start position argument.
    end : Int
        An end position argument.
    step : Int
        A step number argument.

    Returns
    -------
    arr : Array[Int]
        A created array of integers.
    """
    from apysc._expression import expression_data_util

    arr: Array[Int] = Array([])
    expression: str = (
        f"for (var i = {start.variable_name}; i < {end.variable_name}; "
        f"i += {step.variable_name}) {{"
        f"\n  {arr.variable_name}.push(i);"
        "\n}"
    )
    expression_data_util.append_js_expression(expression=expression)
    return arr


def _create_double_args_case_arr(*, start: Int, end: Int) -> Array[Int]:
    """
    Create a double arguments case array.

    Parameters
    ----------
    start : Int
        A start position argument.
    end : Int
        An end position argument.

    Returns
    -------
    arr : Array[Int]
        A created array of integers.
    """
    from apysc._expression import expression_data_util

    arr: Array[Int] = Array([])
    expression: str = (
        f"for (var i = {start.variable_name}; i < {end.variable_name}; i++) {{"
        f"\n  {arr.variable_name}.push(i);"
        "\n}"
    )
    expression_data_util.append_js_expression(expression=expression)
    return arr


def _create_single_arg_case_arr(*, end: Int) -> Array[Int]:
    """
    Create a single argument case array.

    Parameters
    ----------
    end : Int
        An end position argument.

    Returns
    -------
    arr : Array[Int]
        A created array of integers.
    """
    from apysc._expression import expression_data_util

    arr: Array[Int] = Array([])
    expression: str = (
        f"for (var i = 0; i < {end.variable_name}; i++) {{"
        f"\n  {arr.variable_name}.push(i);"
        "\n}"
    )
    expression_data_util.append_js_expression(expression=expression)
    return arr



"""The class implementation for the outlined `settings_backup_restore` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialSettingsBackupRestoreOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `settings_backup_restore` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M14 12c0-1.1-.9-2-2-2s-2 .9-2 2 .9 2 2 2 2-.9 2-2zm-2-9c-4.97 0-9 4.03-9 9H0l4 4 4-4H5c0-3.87 3.13-7 7-7s7 3.13 7 7-3.13 7-7 7c-1.51 0-2.91-.49-4.06-1.3l-1.42 1.44C8.04 20.3 9.94 21 12 21c4.97 0 9-4.03 9-9s-4.03-9-9-9z"/></svg>'  # noqa



"""The class implementation for the `pending` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialPendingIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `pending` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><path d="M12,2C6.48,2,2,6.48,2,12c0,5.52,4.48,10,10,10s10-4.48,10-10C22,6.48,17.52,2,12,2z M7,13.5c-0.83,0-1.5-0.67-1.5-1.5 c0-0.83,0.67-1.5,1.5-1.5s1.5,0.67,1.5,1.5C8.5,12.83,7.83,13.5,7,13.5z M12,13.5c-0.83,0-1.5-0.67-1.5-1.5 c0-0.83,0.67-1.5,1.5-1.5s1.5,0.67,1.5,1.5C13.5,12.83,12.83,13.5,12,13.5z M17,13.5c-0.83,0-1.5-0.67-1.5-1.5 c0-0.83,0.67-1.5,1.5-1.5s1.5,0.67,1.5,1.5C18.5,12.83,17.83,13.5,17,13.5z"/></g></svg>'  # noqa



"""Class implementation for the `apply_current_points` mix-in.
"""

from apysc._display.points_2d_mixin import Points2DMixIn


class PolygonApplyCurrentPointsMixIn(Points2DMixIn):
    def _apply_current_points(self) -> None:
        """
        Apply current points settings and re-draw a polygon graphic.
        """
        from apysc._expression import expression_data_util

        points_var_name: str
        points_expression: str
        points_var_name, points_expression = self._make_2dim_points_expression()
        expression_data_util.append_js_expression(expression=points_expression)
        expression: str = f"{self.variable_name}.plot({points_var_name});"
        expression_data_util.append_js_expression(expression=expression)



"""The class implementation for the `supervisor_account` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialSupervisorAccountIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `supervisor_account` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M16.5 12c1.38 0 2.49-1.12 2.49-2.5S17.88 7 16.5 7C15.12 7 14 8.12 14 9.5s1.12 2.5 2.5 2.5zM9 11c1.66 0 2.99-1.34 2.99-3S10.66 5 9 5C7.34 5 6 6.34 6 8s1.34 3 3 3zm7.5 3c-1.83 0-5.5.92-5.5 2.75V19h11v-2.25c0-1.83-3.67-2.75-5.5-2.75zM9 13c-2.33 0-7 1.17-7 3.5V19h7v-2.25c0-.85.33-2.34 2.37-3.47C10.5 13.1 9.66 13 9 13z"/></svg>'  # noqa



"""The class implementation for the `sentiment_satisfied_alt` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialSentimentSatisfiedAltIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `sentiment_satisfied_alt` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M0 0h24v24H0V0z" fill="none"/><circle cx="15.5" cy="9.5" r="1.5"/><circle cx="8.5" cy="9.5" r="1.5"/><circle cx="15.5" cy="9.5" r="1.5"/><circle cx="8.5" cy="9.5" r="1.5"/><path d="M11.99 2C6.47 2 2 6.48 2 12s4.47 10 9.99 10C17.52 22 22 17.52 22 12S17.52 2 11.99 2zM12 20c-4.42 0-8-3.58-8-8s3.58-8 8-8 8 3.58 8 8-3.58 8-8 8zm0-2.5c2.33 0 4.32-1.45 5.12-3.5h-1.67c-.69 1.19-1.97 2-3.45 2s-2.75-.81-3.45-2H6.88c.8 2.05 2.79 3.5 5.12 3.5z"/></svg>'  # noqa



"""The class implementation for the outlined `app_blocking` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAppBlockingOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `app_blocking` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/><g><path d="M18,8c-2.21,0-4,1.79-4,4c0,2.21,1.79,4,4,4s4-1.79,4-4C22,9.79,20.21,8,18,8z M15.5,12c0-1.38,1.12-2.5,2.5-2.5 c0.42,0,0.8,0.11,1.15,0.29l-3.36,3.36C15.61,12.8,15.5,12.42,15.5,12z M18,14.5c-0.42,0-0.8-0.11-1.15-0.29l3.36-3.36 c0.18,0.35,0.29,0.73,0.29,1.15C20.5,13.38,19.38,14.5,18,14.5z"/><path d="M17,18H7V6h10v1h2V6V5V3c0-1.1-0.9-2-2-2H7C5.9,1,5,1.9,5,3v18c0,1.1,0.9,2,2,2h10c1.1,0,2-0.9,2-2v-2v-1v-1h-2V18z M7,3 h10v1H7V3z M17,21H7v-1h10V21z"/></g></g></svg>'  # noqa



"""Class implementation of any value.
"""

from typing import Any
from typing import Dict
from typing import Optional

from typing_extensions import final

from apysc._event.custom_event_mixin import CustomEventMixIn
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.boolean import Boolean
from apysc._type.copy_mixin import CopyMixIn
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.to_string_mixin import ToStringMixIn
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos


class AnyValue(
    ToStringMixIn,
    CopyMixIn["AnyValue"],
    RevertMixIn,
    CustomEventMixIn,
    VariableNameSuffixMixIn,
):
    """
    Class implementation of any value (a value that can't
    determine type).

    Examples
    --------
    >>> import apysc as ap
    >>> any_value: ap.AnyValue = ap.AnyValue(10)
    >>> any_value.value
    10

    >>> any_value.value = 20
    >>> any_value.value
    20
    """

    _value: Any

    @final
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(self, value: Any, *, variable_name_suffix: str = "") -> None:
        """
        Class implementation of any value (a value that can't
        determine type).

        Parameters
        ----------
        value : *
            Initial any value.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Examples
        --------
        >>> import apysc as ap
        >>> any_value: ap.AnyValue = ap.AnyValue(10)
        >>> any_value.value
        10
        """
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names

        self._variable_name_suffix = variable_name_suffix
        TYPE_NAME: str = var_names.ANY
        self._value = value
        self.variable_name = expression_variables_util.get_next_variable_name(
            type_name=TYPE_NAME
        )
        self._type_name = TYPE_NAME
        self._append_constructor_expression()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_constructor_expression(self) -> None:
        """
        Append constructor expression.
        """
        from apysc._expression import expression_data_util
        from apysc._type import value_util

        expression: str = f"var {self.variable_name} = "
        if isinstance(self._value, VariableNameMixIn):
            expression += f"{self._value.variable_name};"
        else:
            value_str: str = value_util.get_value_str_for_expression(value=self._value)
            expression += f"{value_str};"
        expression_data_util.append_js_expression(expression=expression)

    @property
    @add_debug_info_setting(module_name=__name__)
    def value(self) -> Any:
        """
        Get a current value.

        Returns
        -------
        value : *
            Any value.

        Examples
        --------
        >>> import apysc as ap
        >>> any_value: ap.AnyValue = ap.AnyValue(10)
        >>> any_value.value = 20
        >>> any_value.value
        20
        """
        if isinstance(self._value, CopyMixIn):
            return self._value.copy()
        return self._value

    @value.setter
    @add_debug_info_setting(module_name=__name__)
    def value(self, value: Any) -> None:
        """
        Set a any value.

        Parameters
        ----------
        value : *
            Any value to set.
        """
        self._value = value
        self._append_value_setter_expression(value=value)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_value_setter_expression(self, *, value: Any) -> None:
        """
        Append value's setter expression.

        Parameters
        ----------
        value : *
            Any value to set.
        """
        from apysc._expression import expression_data_util

        expression: str = f"{self.variable_name} = "
        if isinstance(value, VariableNameMixIn):
            expression += f"{value.variable_name};"
        else:
            expression += f"{value};"
        expression_data_util.append_js_expression(expression=expression)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_arithmetic_operation_expression(
        self, *, other: Any, operator: str
    ) -> VariableNameMixIn:
        """
        Append arithmetic operation (e.g., addition) expression.

        Parameters
        ----------
        other : Any
            Other value to use.
        operator : str
            JavaScript arithmetic operator, like '+', '*', and so on.

        Returns
        -------
        result : AnyValue
            Calculated result value.
        """
        from apysc._expression import expression_data_util
        from apysc._type.value_util import get_value_str_for_expression

        value_str: str = get_value_str_for_expression(value=other)
        result: AnyValue = self.copy()
        expression: str = (
            f"{result.variable_name} = " f"{self.variable_name} {operator} {value_str};"
        )
        expression_data_util.append_js_expression(expression=expression)
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def __add__(self, other: Any) -> Any:
        """
        Method for addition.

        Parameters
        ----------
        other : Any
            Other value to add.

        Returns
        -------
        result : AnyValue
            Addition result value.
        """
        result: VariableNameMixIn = self._append_arithmetic_operation_expression(
            other=other, operator="+"
        )
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def __sub__(self, other: Any) -> Any:
        """
        Method for subtraction.

        Parameters
        ----------
        other : Any
            The other value to subtract.

        Returns
        -------
        result : AnyValue
            Subtraction result value.
        """
        result: VariableNameMixIn = self._append_arithmetic_operation_expression(
            other=other, operator="-"
        )
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def __mul__(self, other: Any) -> Any:
        """
        Method for multiplication.

        Parameters
        ----------
        other : Any
            Other value to multiply.

        Returns
        -------
        result : AnyValue
            Subtraction result value.
        """
        result: VariableNameMixIn = self._append_arithmetic_operation_expression(
            other=other, operator="*"
        )
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def __truediv__(self, other: Any) -> Any:
        """
        Method for true division.

        Parameters
        ----------
        other : Any
            Other value for true division.

        Returns
        -------
        result : AnyValue
            True division result value.
        """
        result: VariableNameMixIn = self._append_arithmetic_operation_expression(
            other=other, operator="/"
        )
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def __floordiv__(self, other: Any) -> Any:
        """
        Method for floor division.

        Parameters
        ----------
        other : Any
            Other value for floor division.

        Returns
        -------
        result : AnyValue
            Floor division result value.
        """
        from apysc._expression import expression_data_util
        from apysc._type.value_util import get_value_str_for_expression

        result: AnyValue = self.copy()
        value_str: str = get_value_str_for_expression(value=other)
        expression: str = (
            f"{result.variable_name} = "
            f"Math.trunc({self.variable_name} / {value_str});"
        )
        expression_data_util.append_js_expression(expression=expression)
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_incremental_arithmetic_operation_expression(
        self, *, other: Any, operator: str
    ) -> None:
        """
        Append incremental arithmetic operation (e.g., incremental
        addition) expression.

        Parameters
        ----------
        other : Any
            Other value to use.
        operator : str
            JavaScript arithmetic operator, like '+=', '*=', and so on.
        """
        from apysc._expression import expression_data_util
        from apysc._type.value_util import get_value_str_for_expression

        value_str: str = get_value_str_for_expression(value=other)
        expression: str = f"{self.variable_name} {operator} {value_str};"
        expression_data_util.append_js_expression(expression=expression)

    @final
    @add_debug_info_setting(module_name=__name__)
    def __iadd__(self, other: Any) -> Any:
        """
        Method for incremental addition.

        Parameters
        ----------
        other : Any
            Other value for incremental addition.

        Returns
        -------
        result : AnyValue
            Incremental addition result value.
        """
        self._append_incremental_arithmetic_operation_expression(
            other=other, operator="+="
        )
        return self

    @final
    @add_debug_info_setting(module_name=__name__)
    def __isub__(self, other: Any) -> Any:
        """
        Method for incremental subtraction.

        Parameters
        ----------
        other : Any
            Other value for incremental subtraction.

        Returns
        -------
        result : AnyValue
            Incremental subtraction result value.
        """
        self._append_incremental_arithmetic_operation_expression(
            other=other, operator="-="
        )
        return self

    @final
    @add_debug_info_setting(module_name=__name__)
    def __imul__(self, other: Any) -> Any:
        """
        Method for incremental multiplication.

        Parameters
        ----------
        other : Any
            Other value for incremental multiplication.

        Returns
        -------
        result : AnyValue
            Incremental multiplication result value.
        """
        self._append_incremental_arithmetic_operation_expression(
            other=other, operator="*="
        )
        return self

    @final
    @add_debug_info_setting(module_name=__name__)
    def __itruediv__(self, other: Any) -> Any:
        """
        Method for incremental true division.

        Parameters
        ----------
        other : Any
            Other value for incremental division.

        Returns
        -------
        result : AnyValue
            Incremental division result value.
        """
        self._append_incremental_arithmetic_operation_expression(
            other=other, operator="/="
        )
        return self

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_comparison_expression(
        self, *, comparison_operator: str, other: Any
    ) -> Boolean:
        """
        Append comparison operation expression.

        Parameters
        ----------
        comparison_operator : str
            JavaScript comparison operator (e.g., '===', '>=',
            and so on).
        other : Any
            The other value to compare.

        Returns
        -------
        result : Boolean
            Comparison result. This value always becomes
            False on Python since the correct comparison
            is impossible.
        """
        from apysc._expression import expression_data_util
        from apysc._type.value_util import get_value_str_for_expression

        result: Boolean = Boolean(False)
        value_str: str = get_value_str_for_expression(value=other)
        expression: str = (
            f"{result.variable_name} = "
            f"{self.variable_name} {comparison_operator} {value_str};"
        )
        expression_data_util.append_js_expression(expression=expression)
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def __eq__(self, other: Any) -> Any:
        """
        Equal comparison method.

        Parameters
        ----------
        other : Any
            The other value to compare.

        Returns
        -------
        result : Boolean
            Comparison result. This value always becomes
            False on Python since the correct comparison
            is impossible.
        """
        result: Boolean = self._append_comparison_expression(
            comparison_operator="===", other=other
        )
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def __ne__(self, other: Any) -> Any:
        """
        Not equal comparison method.

        Parameters
        ----------
        other : Any
            The other value to compare.

        Returns
        -------
        result : Boolean
            Comparison result. This value always becomes
            False on Python since the correct comparison
            is impossible.
        """
        result: Boolean = self._append_comparison_expression(
            comparison_operator="!==", other=other
        )
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def __lt__(self, other: Any) -> Boolean:
        """
        Less than comparison method.

        Parameters
        ----------
        other : Any
            The other value to compare.

        Returns
        -------
        result : Boolean
            Comparison result. This value always becomes
            False on Python since the correct comparison
            is impossible.
        """
        result: Boolean = self._append_comparison_expression(
            comparison_operator="<", other=other
        )
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def __le__(self, other: Any) -> Boolean:
        """
        Less than equal comparison method.

        Parameters
        ----------
        other : Any
            The other value to compare.

        Returns
        -------
        result : Boolean
            Comparison result. This value always becomes
            False on Python since the correct comparison
            is impossible.
        """
        result: Boolean = self._append_comparison_expression(
            comparison_operator="<=", other=other
        )
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def __gt__(self, other: Any) -> Boolean:
        """
        Greater than comparison method.

        Parameters
        ----------
        other : Any
            The other value to compare.

        Returns
        -------
        result : Boolean
            Comparison result. This value always becomes
            False on Python since the correct comparison
            is impossible.
        """
        result: Boolean = self._append_comparison_expression(
            comparison_operator=">", other=other
        )
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def __ge__(self, other: Any) -> Boolean:
        """
        Greater than equal comparison method.

        Parameters
        ----------
        other : Any
            The other value to compare.

        Returns
        -------
        result : Boolean
            Comparison result. This value always becomes
            False on Python since the correct comparison is impossible.
        """
        result: Boolean = self._append_comparison_expression(
            comparison_operator=">=", other=other
        )
        return result

    _any_value_snapshots: Optional[Dict[str, Any]] = None

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make value's snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._set_single_snapshot_val_to_dict(
            dict_name="_any_value_snapshots",
            value=self._value,
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert value if snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._value = self._get_snapshot_val_if_exists(
            current_value=self._value,
            snapshot_dict=self._any_value_snapshots,
            snapshot_name=snapshot_name,
        )



"""This module is for the translation mapping data of the
following document:

Document file: math_trunc.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Math trunc interface": "# Math クラスの trunc インターフェイス",
    ##################################################
    "This page explains the `Math` class's `trunc` class method interface.": "このページでは`Math`クラスの`trunc`クラスメソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `trunc` class method interface truncates a fraction value and returns an integer value.": "`trunc`クラスメソッドのインターフェイスは指定された値の小数点以下の値を切り捨てて整数の値を返却します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `trunc` interface requires `Int`, `Number`, `String`, or `Boolean` value argument.": "`trunc`インターフェイスは`Int`、`Number`、`String`、もしくは`Boolean`のいずれかの型の値の引数を必要とします。",  # noqa
    ##################################################
    "If an argument is a `Number` or `String` value, this interface truncates a fraction value and converts it to the `Int` type.": "もしも引数の値が`Number`もしくは`String`型の値であればこのインターフェイスは小数点以下の値を切り捨てて`Int`型に変換した状態で返却します。",  # noqa
    ##################################################
    "If an argument is a `Boolean` value, this interface returns 0 or 1 `Int` value.": "もしも引数の値が`Boolean`の型の値であれば、このインターフェイスは0もしくは1の`Int`型の値を返却します。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nresult_int: ap.Int = ap.Math.trunc(value=ap.Int(10))\nassert result_int == 10\n\nresult_int = ap.Math.trunc(value=ap.Number(8.5))\nassert result_int == 8\n\nresult_int = ap.Math.trunc(value=ap.String("7.6"))\nassert result_int == 7\n\nresult_int = ap.Math.trunc(value=ap.Boolean(True))\nassert result_int == 1\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nresult_int: ap.Int = ap.Math.trunc(value=ap.Int(10))\nassert result_int == 10\n\nresult_int = ap.Math.trunc(value=ap.Number(8.5))\nassert result_int == 8\n\nresult_int = ap.Math.trunc(value=ap.String("7.6"))\nassert result_int == 7\n\nresult_int = ap.Math.trunc(value=ap.Boolean(True))\nassert result_int == 1\n```',  # noqa
    ##################################################
    "## Math.trunc API": "## Math.trunc のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Truncate a fraction value from a specified value.<hr>": "指定された値から小数点以下の値を切り落とします。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `value`: Union[Int, Number, String, Boolean]": "- `value`: Union[Int, Number, String, Boolean]",  # noqa
    ##################################################
    "  - A value to truncate a fraction value. If a specified value is the `Number`'s, `String`'s, or `Boolean`'s type, the return value becomes an `Int`'s value.": "  - 小数点以下を切り捨てる対象の値。もし指定された値が`Number`、`String`、もしくは`Boolean`型の値の場合、変逆値は`Int`型の値となります。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `result`: Int": "- `result`: Int",
    ##################################################
    "  - Truncated and converted to `Int`'s value.": "  - 切り捨て処理と`Int`型への変換が反映された値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> result_int: ap.Int = ap.Math.trunc(value=ap.Int(10))\n>>> result_int\nInt(10)\n\n>>> result_int = ap.Math.trunc(value=ap.Number(8.5))\n>>> result_int\nInt(8)\n\n>>> result_int = ap.Math.trunc(value=ap.String("7.6"))\n>>> result_int\nInt(7)\n\n>>> result_int = ap.Math.trunc(value=ap.Boolean(True))\n>>> result_int\nInt(1)\n\n>>> result_int = ap.Math.trunc(value=ap.Boolean(False))\n>>> result_int\nInt(0)\n```': '```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> result_int: ap.Int = ap.Math.trunc(value=ap.Int(10))\n>>> result_int\nInt(10)\n\n>>> result_int = ap.Math.trunc(value=ap.Number(8.5))\n>>> result_int\nInt(8)\n\n>>> result_int = ap.Math.trunc(value=ap.String("7.6"))\n>>> result_int\nInt(7)\n\n>>> result_int = ap.Math.trunc(value=ap.Boolean(True))\n>>> result_int\nInt(1)\n\n>>> result_int = ap.Math.trunc(value=ap.Boolean(False))\n>>> result_int\nInt(0)\n```',  # noqa
}



"""The class implementation for the `wifi_calling` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialWifiCallingIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `wifi_calling` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><path d="M22,4.95C21.79,4.78,19.67,3,16.5,3c-3.18,0-5.29,1.78-5.5,1.95L16.5,12L22,4.95z"/><path d="M20,15.51c-1.24,0-2.45-0.2-3.57-0.57c-0.35-0.12-0.75-0.03-1.02,0.24l-2.2,2.2c-2.83-1.45-5.15-3.76-6.59-6.59l2.2-2.2 C9.1,8.31,9.18,7.92,9.07,7.57C8.7,6.45,8.5,5.25,8.5,4c0-0.55-0.45-1-1-1H4C3.45,3,3,3.45,3,4c0,9.39,7.61,17,17,17 c0.55,0,1-0.45,1-1v-3.49C21,15.96,20.55,15.51,20,15.51z"/></g></g></svg>'  # noqa



"""The mix-in class implementation for the
`_append_line_alpha_attr_expression` method.
"""

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting


class AppendLineAlphaAttrExpressionMixIn:
    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_line_alpha_attr_expression(
        self,
        *,
        expression: str,
        indent_num: int,
        skip_appending: bool = False,
    ) -> str:
        """
        Append line-alpha attribute expression to a specified
        expression string.

        Parameters
        ----------
        expression : str
            A target expression string.
        indent_num : int
            An indentation number.
        skip_appending : bool, optional
            A boolean, whether to skip appending.

        Returns
        -------
        expression : str
            After an appending expression string.

        Raises
        ------
        TypeError
            If this is not a `LineAlphaMixIn` instance.
        """
        from apysc._display import graphics_expression
        from apysc._display.line_alpha_mixin import LineAlphaMixIn

        if not isinstance(self, LineAlphaMixIn):
            raise TypeError(
                f"This instance is not a `{LineAlphaMixIn.__name__}` instance: "
                f"{type(self).__name__}"
            )
        if skip_appending:
            return expression

        self._initialize_line_alpha_if_not_initialized()
        expression = graphics_expression.append_stroke_opacity_expression(
            line_alpha=self._line_alpha,
            expression=expression,
            indent_num=indent_num,
        )
        return expression



"""The mix-in class implementation for the `_set_initial_matrix_data` method.
"""

from typing import Dict
from typing import List
from typing import Union
from typing import cast

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.array import Array
from apysc._type.dictionary import Dictionary
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.string import String
from apysc._validation import arg_validation_decos

_DataType = Union[
    Array[Dictionary[String, Union[Int, Number, String]]],
    List[Dict[str, Union[int, float, str]]],
]


class SetInitialMatrixDataMixIn:
    _matrix_data: Array[Dictionary[String, Union[Int, Number, String]]]

    @final
    @arg_validation_decos.is_list_or_array_matrix_data(arg_position_index=1)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_matrix_data(
        self,
        *,
        data: _DataType,
        variable_name_suffix: str,
    ) -> None:
        """
        Set an initial matrix data.

        Parameters
        ----------
        data : _DataType
            A data array, which contains a 1-dimensional string key dictionary.
            A list of dictionaries or an `ap.Array` of `ap.Dictionary` values
            are acceptable.
            E.g., `[{"column_name_1": 10, "column_name_2"}]`
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        """
        if not isinstance(data, Array):
            data_: Array[Dictionary[String, Union[Int, Number, String]]] = (
                _convert_list_to_array(
                    data=cast(List[Dict[str, Union[int, float, str]]], data),
                    variable_name_suffix=variable_name_suffix,
                )
            )
        else:
            data_ = data
        self._matrix_data = data_


def _convert_list_to_array(
    *,
    data: List[Dict[str, Union[int, float, str]]],
    variable_name_suffix: str,
) -> Array[Dictionary[String, Union[Int, Number, String]]]:
    """
    Convert a specified matrix list to an array.

    Parameters
    ----------
    data : List[Dict[str, Union[int, float, str]]]
        A matrix list.
    variable_name_suffix : str, optional
        A JavaScript variable name suffix string.
        This setting is sometimes useful for JavaScript debugging.

    Returns
    -------
    data__ : Array[Dictionary[String, Union[Int, Number, String]]]:
        A converted array.
    """
    data_: List[Dictionary[String, Union[Int, Number, String]]] = []
    for dict_data in data:
        dict_data_: Dict[String, Union[Int, Number, String]] = {}
        for key, value in dict_data.items():
            key_: String = String(key, variable_name_suffix=variable_name_suffix)
            if isinstance(value, int):
                dict_data_[key_] = Int(value, variable_name_suffix=variable_name_suffix)
                continue
            if isinstance(value, float):
                dict_data_[key_] = Number(
                    value, variable_name_suffix=variable_name_suffix
                )
                continue
            if isinstance(value, str):
                dict_data_[key_] = String(
                    value, variable_name_suffix=variable_name_suffix
                )
                continue
        data_.append(Dictionary(dict_data_, variable_name_suffix=variable_name_suffix))
    data__: Array[Dictionary[String, Union[Int, Number, String]]] = Array(
        data_, variable_name_suffix=variable_name_suffix
    )
    return data__



"""This module is for the translation mapping data of the
following document:

Document file: string_addition_and_multiplication.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# String class addition and multiplication operations": "# String クラスの加算と乗算の各オペレーション",  # noqa
    ##################################################
    "This page explains the `String` class addition and multiplication operations.": "このページでは`String`クラスの加算と乗算のオペレーションについて説明します。",  # noqa
    ##################################################
    "## Addition": "## 加算",
    ##################################################
    "The `String` class addition operation (`+`) returns the concatenated `String` value:": "`String`クラスの加算のオペレーション（`+`）は連結された`String`型の値を返却します。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nstring_1: ap.String = ap.String("Hello")\nstring_2: ap.String = string_1 + " World!"\nassert string_2 == "Hello World!"\nassert isinstance(string_2, ap.String)\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nstring_1: ap.String = ap.String("Hello")\nstring_2: ap.String = string_1 + " World!"\nassert string_2 == "Hello World!"\nassert isinstance(string_2, ap.String)\n```',  # noqa
    ##################################################
    "Also, the `+=` operator is supported:": "また、`+=`のオペレーターもサポートしています:",
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nstring_1: ap.String = ap.String("Hello")\nstring_1 += " World!"\nassert string_1 == "Hello World!"\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nstring_1: ap.String = ap.String("Hello")\nstring_1 += " World!"\nassert string_1 == "Hello World!"\n```',  # noqa
    ##################################################
    "A `String` value + Python built-in `str` operation is supported. Similarly, a `String` value + `String` value operation is also supported:": "`String`の値とPythonのビルトインの`str`の値によるオペレーションもサポートしています。`String`の値同士のオペレーションも同様です。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nstring_1: ap.String = ap.String("Hello")\nstring_2: ap.String = ap.String(" World!")\nstring_3: ap.String = string_1 + string_2\nassert string_3 == "Hello World!"\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nstring_1: ap.String = ap.String("Hello")\nstring_2: ap.String = ap.String(" World!")\nstring_3: ap.String = string_1 + string_2\nassert string_3 == "Hello World!"\n```',  # noqa
    ##################################################
    "But a Python built-in `str` + `String` value is not supported; for instance, the following code raises an error:": "一方で`str`の値と`String`の値の場合（左側を`str`の値にする場合）はサポートしていません。例えば以下のコード例ではエラーとなります:",  # noqa
    ##################################################
    '```py\nimport apysc as ap\n\nap.Stage()\nstring_1: ap.String = ap.String(" World!")\nstring_2: ap.String = "Hello" + string_1\n```': '```py\nimport apysc as ap\n\nap.Stage()\nstring_1: ap.String = ap.String(" World!")\nstring_2: ap.String = "Hello" + string_1\n```',  # noqa
    ##################################################
    "```\nTypeError: must be str, not String\n```": "```\nTypeError: must be str, not String\n```",  # noqa
    ##################################################
    "## Multiplication": "## 乗算",
    ##################################################
    "The `String` class multiplication operation (`*`) returns the repeated `String` value, same behaviors as the Python built-in `str` value:": "`String`クラスの乗算のオペレーション（`*`）はPythonビルトインの文字列のように値を繰り返した文字列を返却します。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nstring_1: ap.String = ap.String("Hello")\nstring_2: ap.String = string_1 * 3\nassert string_2 == "HelloHelloHello"\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nstring_1: ap.String = ap.String("Hello")\nstring_2: ap.String = string_1 * 3\nassert string_2 == "HelloHelloHello"\n```',  # noqa
    ##################################################
    "The `int` or `Int` values are acceptable at the operation's right-side value:": "`int`もしくは`Int`型の値を右側の値として受け付けることができます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nstring_1: ap.String = ap.String("Hello")\nint_1: ap.Int = ap.Int(3)\nstring_2: ap.String = string_1 * int_1\nassert string_2 == "HelloHelloHello"\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nstring_1: ap.String = ap.String("Hello")\nint_1: ap.Int = ap.Int(3)\nstring_2: ap.String = string_1 * int_1\nassert string_2 == "HelloHelloHello"\n```',  # noqa
}



"""The class implementation for the `face` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialFaceIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `face` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M9 11.75c-.69 0-1.25.56-1.25 1.25s.56 1.25 1.25 1.25 1.25-.56 1.25-1.25-.56-1.25-1.25-1.25zm6 0c-.69 0-1.25.56-1.25 1.25s.56 1.25 1.25 1.25 1.25-.56 1.25-1.25-.56-1.25-1.25-1.25zM12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm0 18c-4.41 0-8-3.59-8-8 0-.29.02-.58.05-.86 2.36-1.05 4.23-2.98 5.21-5.37C11.07 8.33 14.05 10 17.42 10c.78 0 1.53-.09 2.25-.26.21.71.33 1.47.33 2.26 0 4.41-3.59 8-8 8z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: graphics_draw_ellipse.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Graphics draw_ellipse interface": "# Graphics クラスの draw_ellipse インターフェイス",
    ##################################################
    "This page explains the `Graphics` class `draw_ellipse` method interface.": "このページでは`Graphics`クラスの`draw_ellipse`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `draw_ellipse` interface draws the vector ellipse graphics.": "`draw_ellipse`インターフェイスは楕円のベクターグラフィックスを描画します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `draw_ellipse` interface has the `x`\\, `y`\\, `width`\\, and `height` arguments. The `x` and `y` arguments are the ellipse center coordinates. The `width` and `height` arguments are the ellipse size. These sizes are twice the size of the radius.": "`draw_ellipse`インターフェイスは`x`、`y`、`width`、`height`の各インターフェイスを持っています。`x`と`y`の引数は楕円の中央座標となります。`width`と`height`は楕円の幅と高さを決定します。これらのサイズは半径の倍の値（直径）で指定する必要があります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=325,\n    stage_height=200,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\n\n# Set the cyan fill color and draw the ellipse.\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nsprite.graphics.draw_ellipse(x=125, y=100, width=150, height=100)\n\n# Set the only dotted-line style and draw the ellipse.\nsprite.graphics.begin_fill(color=ap.COLORLESS)\nsprite.graphics.line_style(\n    color=ap.Color("#fff"),\n    thickness=3,\n    dot_setting=ap.LineDotSetting(dot_size=3),\n)\nsprite.graphics.draw_ellipse(x=200, y=100, width=150, height=100)\n\nap.save_overall_html(dest_dir_path="graphics_draw_ellipse_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=325,\n    stage_height=200,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\n\n# Set the cyan fill color and draw the ellipse.\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nsprite.graphics.draw_ellipse(x=125, y=100, width=150, height=100)\n\n# Set the only dotted-line style and draw the ellipse.\nsprite.graphics.begin_fill(color=ap.COLORLESS)\nsprite.graphics.line_style(\n    color=ap.Color("#fff"),\n    thickness=3,\n    dot_setting=ap.LineDotSetting(dot_size=3),\n)\nsprite.graphics.draw_ellipse(x=200, y=100, width=150, height=100)\n\nap.save_overall_html(dest_dir_path="graphics_draw_ellipse_basic_usage/")\n```',  # noqa
    ##################################################
    "## Return value": "## 返却値",
    ##################################################
    "The return value of the `draw_ellipse` interface is the instance of the `Ellipse` class.": "`draw_ellipse`インターフェイスの返却値は`Ellipse`クラスのインスタンスとなります。",  # noqa
    ##################################################
    "It has basic interfaces (like the `x` or the `width` attributes) similar to the other graphics classes.": "このクラスのインスタンスは他のグラフィックス系のクラスと同様に`x`や`y`、`width`などの基本的なインターフェイスを持っています。",  # noqa
    ##################################################
    "The following code example binds the click event handler. If you click the ellipse, the width and height become wider.": "以下のコード例ではクリックのイベントハンドラを設定しており、楕円をクリックするたびに幅と高さが大きくなるようにしています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_ellipse_click(e: ap.MouseEvent[ap.Ellipse], options: dict) -> None:\n    """\n    The handler that the ellipse calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ellipse: ap.Ellipse = e.this\n    ellipse.width += 15\n    ellipse.height += 10\n\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=250,\n    stage_height=200,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nellipse: ap.Ellipse = sprite.graphics.draw_ellipse(x=125, y=100, width=150, height=100)\nellipse.click(on_ellipse_click)\n\nap.save_overall_html(dest_dir_path="graphics_draw_ellipse_return_value/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_ellipse_click(e: ap.MouseEvent[ap.Ellipse], options: dict) -> None:\n    """\n    The handler that the ellipse calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ellipse: ap.Ellipse = e.this\n    ellipse.width += 15\n    ellipse.height += 10\n\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=250,\n    stage_height=200,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nellipse: ap.Ellipse = sprite.graphics.draw_ellipse(x=125, y=100, width=150, height=100)\nellipse.click(on_ellipse_click)\n\nap.save_overall_html(dest_dir_path="graphics_draw_ellipse_return_value/")\n```',  # noqa
    ##################################################
    "## draw_ellipse API": "## draw_ellipse API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Draw an ellipse vector graphic.<hr>": "楕円のベクターグラフィックスを描画します。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `x`: float or Number": "- `x`: float or Number",
    ##################################################
    "  - X-coordinate of the ellipse center.": "  - 楕円の中央のX座標。",
    ##################################################
    "- `y`: float or Number": "- `y`: float or Number",
    ##################################################
    "  - Y-coordinate of the ellipse center.": "  - 楕円の中央のY座標。",
    ##################################################
    "- `width`: Int or int": "- `width`: Int or int",
    ##################################################
    "  - Ellipse width.": "  - 楕円の幅。",
    ##################################################
    "- `height`: Int or int": "- `height`: Int or int",
    ##################################################
    "  - Ellipse height.": "  - 楕円の高さ。",
    ##################################################
    '- `variable_name_suffix`: str, default ""': '- `variable_name_suffix`: str, default ""',  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `ellipse`: Ellipse": "- `ellipse`: Ellipse",
    ##################################################
    "  - Created ellipse graphics instance.": "  - 作成された楕円のグラフィックスのインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> ellipse: ap.Ellipse = sprite.graphics.draw_ellipse(\n...     x=100, y=100, width=100, height=50\n... )\n>>> ellipse.x\nNumber(100.0)\n\n>>> ellipse.y\nNumber(100.0)\n\n>>> ellipse.width\nInt(100)\n\n>>> ellipse.height\nInt(50)\n\n>>> ellipse.fill_color\nColor("#00aaff")\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> ellipse: ap.Ellipse = sprite.graphics.draw_ellipse(\n...     x=100, y=100, width=100, height=50\n... )\n>>> ellipse.x\nNumber(100.0)\n\n>>> ellipse.y\nNumber(100.0)\n\n>>> ellipse.width\nInt(100)\n\n>>> ellipse.height\nInt(50)\n\n>>> ellipse.fill_color\nColor("#00aaff")\n```',  # noqa
}



"""Mix-in class implementation for the `_set_initial_border_alpha` method.
"""

from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.number import Number
from apysc._validation import arg_validation_decos


class SetInitialBorderAlphaMixIn:
    _border_alpha: Number

    @final
    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    @arg_validation_decos.num_is_0_to_1_range(arg_position_index=1, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_border_alpha(
        self,
        *,
        border_alpha: Union[float, Number],
        variable_name_suffix: str = "",
    ) -> None:
        """
        Set a chart's initial border alpha.

        Parameters
        ----------
        border_alpha : Union[float, Number]
            A chart's border alpha.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        """
        if not isinstance(border_alpha, Number):
            border_alpha_: Number = Number(
                border_alpha, variable_name_suffix=variable_name_suffix
            )
        else:
            border_alpha_ = border_alpha
        self._border_alpha = border_alpha_



"""This module is for the translation mapping data of the
following document:

Document file: display_object_x_and_y.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# DisplayObject x and y interfaces": "# DisplayObject クラスの x と y のインターフェイス",
    ##################################################
    "This page explains the `DisplayObject` class x and y property interfaces.": "このページでは`DisplayObject`クラスのxとy属性のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The x and y properties change the `DisplayObject` instance 2-dimensional coordinates.": "xとy属性は`DisplayObject`のインスタンスの2次元の座標位置を変更します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "Each `DisplayObject` instance has the x and y properties and can get/set the value with it.": "`DisplayObject`の各インスタンスはxとy属性を持っており、それを参照して座標値の取得と更新を行うことができます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=0, y=0, width=50, height=50)\n\n# Update the x and y coordinates from 0 to 50.\nrectangle.x = ap.Number(50)\nrectangle.y = ap.Number(50)\n\nap.save_overall_html(dest_dir_path="display_object_x_and_y_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=0, y=0, width=50, height=50)\n\n# Update the x and y coordinates from 0 to 50.\nrectangle.x = ap.Number(50)\nrectangle.y = ap.Number(50)\n\nap.save_overall_html(dest_dir_path="display_object_x_and_y_basic_usage/")\n```',  # noqa
    ##################################################
    "## Augmented assignment": "## 累算代入演算",
    ##################################################
    "The x and y properties support the Augmented assignments, like the `+=`\\, `-=`\\, `/=`\\, and `*=` operators.": "xとy属性は`+=`や`-=`、`/=`、`*=`の累算代入演算をサポートしています。",  # noqa
    ##################################################
    "The following example appends 10-pixel to the y-coordinate when you click the rectangle.": "以下のコード例では四角をクリックする度に10pxずつY座標を加算しています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.y += 10\n\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="display_object_x_and_y_augmented_assignment/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.y += 10\n\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="display_object_x_and_y_augmented_assignment/")\n```',  # noqa
    ##################################################
    "## x property API": "## x属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get an x-coordinate.<hr>": "X座標を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `x`: Number": "- `x`: Number",
    ##################################################
    "  - X-coordinate.": "  - X座標。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle.x = ap.Number(100)\n>>> rectangle.x\nNumber(100.0)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle.x = ap.Number(100)\n>>> rectangle.x\nNumber(100.0)\n```',  # noqa
    ##################################################
    "## y property API": "## y属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a y-coordinate.<hr>": "Y座標を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `y`: Number": "- `y`: Number",
    ##################################################
    "  - Y-coordinate.": "  - Y座標。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle.y = ap.Number(100)\n>>> rectangle.y\nNumber(100.0)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle.y = ap.Number(100)\n>>> rectangle.y\nNumber(100.0)\n```',  # noqa
}
